% Inner loop of shock identification

% ---------------------------------------------------------------------
% GET BOUNDS via Plug in approach for all draws
% ---------------------------------------------------------------------

% =====================================================================
% Nummerical Bounds via Sign Restrictions
% =====================================================================


SOLUTION_POSSIBLE = false; % flag within the fmincon loop
StopComputingRes=false;
this_horz=0;
[xfeasible, cheb_r, cheb_exit_flag ] = recursive_chebycenter(AnyRestrictionsA, AnyRestrictions_combined);
% if sufficient condition fails, try numerical solution
if cheb_exit_flag~=1 && cheb_exit_flag~=3
    StopComputingRes = true;
    fprintf(2,'cannot find a feasible starting point \n');
end;
    
if ~StopComputingRes  
    starting_x=xfeasible;
    t=cputime;
    try
        x_draws_full = rmvnrnd_mdims(ndraws_q*GibbsStep,ceil(ndraws_q*GibbsStep/3),nvar,nshocks,AnyRestrictionsA, starting_x);
        if all(all(x_draws_full(:,:,end)==0))
            error('Algorithm 2 failed');
        end;
        x_draws = x_draws_full(:,:,1:GibbsStep:end);
        chol_irf=zeros(nvar,nvar,nstep);
        chol_irf(:,:,1)=setup_irf(biga,tmp_chol,J,1, 0);

        Total_FEVD=zeros(nvar,nstep);
        Total_FEVD(:,1) = diag(chol_irf(:,:,1)*chol_irf(:,:,1)');
        
        for hh=2:nstep
            chol_irf(:,:,hh)=Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,hh, 1) + ...
                (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,hh, 0) ;
 
                Total_FEVD(:,hh) = Total_FEVD(:,hh-1)+ diag(chol_irf(:,:,hh)*chol_irf(:,:,hh)');
 
        end;

        for n=1:ndraws_q
            for shock_idx=1:nshocks
                x_draws(:,shock_idx,n)=x_draws(:,shock_idx,n)/norm(x_draws(:,shock_idx,n));
                for hh=1:nstep
                    q=x_draws(:,shock_idx,n);
                    if hh>1
                        AnyR_FEVD_q(:, hh, shock_idx,n)=AnyR_FEVD_q(:, hh-1, shock_idx,n)+diag(chol_irf(:,:,hh)*q*q'*chol_irf(:,:,hh)');
                    else
                        AnyR_FEVD_q(:, hh, shock_idx,n)=diag(chol_irf(:,:,hh)*q*q'*chol_irf(:,:,hh)');
                    end
                end;
            end;
            AnyR_FEVD_q(:, :, :,n)=AnyR_FEVD_q(:, :, :,n)./ repmat(Total_FEVD,[1,1,nshocks]);
        end;
        % compute benchmark for Algorithm 1
        for hh_pos=1:nstep_fevd
            hh=FEVD_horizons(hh_pos);
            for shock_idx=1:nshocks
                [AnyR_FEVD_bound_q(:,hh_pos,shock_idx,1), idx]=min(AnyR_FEVD_q(:, hh, shock_idx,:),[],4);
                AnyR_qext_FEVD_q(:, hh_pos,:,1,shock_idx)=reshape( reshape(x_draws(:,:,idx(FEVD_variables_num)), [nvar*nshocks, nvar_fevd])', [nvar_fevd, 1, nvar*nshocks]);

                [AnyR_FEVD_bound_q(:,hh_pos,shock_idx,2), idx]=max(AnyR_FEVD_q(:, hh, shock_idx,:),[],4);
                AnyR_qext_FEVD_q(:, hh_pos,:,2,shock_idx)=reshape( reshape(x_draws(:,:,idx(FEVD_variables_num)), [nvar*nshocks, nvar_fevd])', [nvar_fevd, 1, nvar*nshocks]);
            end;
        end;
    catch ME
        StopComputingRes = true;
        fprintf(num2str(draw_i))
        fprintf(2, ' :: problem with Algorithm 2 - perhaps restrictions too tight or empty? \n');
    end;
end;

if DO_VERBOSE && nshocks>1
    disp('Test of Alg 2 (correlation / largest value of inequality constraint):');
    for nn=1:nshocks-1
        disp([max(abs(reshape(sum(squeeze(x_draws(:,1,:)).*squeeze(x_draws(:,nn+1,:))/9), 1, []))) max(max(AnyRestrictions_combined*reshape(x_draws, nvar*nshocks, ndraws_q),[],2))]);
    end;
end;

new_column = zeros(nvar,1);

hh_pos=0;
while hh_pos < nstep_fevd && ~StopComputingRes
    hh_pos=hh_pos+1;
    this_horz=FEVD_horizons(hh_pos);
    
    shock_idx = 0;
    while shock_idx < nshocks  && ~StopComputingRes 
        shock_idx = shock_idx +1;
        sel_var=0;
        while sel_var < nvar_fevd && ~StopComputingRes 
            sel_var=sel_var+1;
            if DO_VERBOSE
                fprintf([num2str([this_horz, shock_idx, sel_var], ':: %1.2f') '\n']);
            end;
            tmp_irf=setup_irf(biga,tmp_chol,J,this_horz, Do_Cumulative(sel_var, 1));

            % -----------------------------------------------------------------
            % IRF minimization part
            % -----------------------------------------------------------------
            sel_vec         = zeros(1,nvar);
            sel_vec(FEVD_variables_num(sel_var))= 1;

            ii=0;
            min_exit_flag=-1;
            if cheb_exit_flag~=1
                fprintf(2, 'Chebychev center sufficient condition failed \n');
            end;
            % warm start to optimization problem unless first horizon
            if hh_pos==1 || ~DO_ALG2_START
                % repmat unnecessary in modern Matlab, but cleaner
                x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);
            else
                if ~DO_ALG2_START
                    x0_hr = reshape(AnyR_qext_FEVD_q(FEVD_variables_num(sel_var), hh_pos, :, shock_idx, 1), nvar, nshocks);
                    x0_hr = x0_hr./repmat(sqrt(sum(x0_hr.^2,1)), nvar,1);
                else
                    if ~any(isnan(AnyR_q(FEVD_variables_num(sel_var),hh_pos-1,:, 1, shock_idx))) && (DO_CHEB_START)
                        x0_hr = squeeze(AnyR_q(FEVD_variables_num(sel_var),hh_pos-1,:, 1, shock_idx));
                        [~,eq_const_val]=my_eq_cons(x0_hr(:));
                        if eq_const_val>CommonTol
                            fprintf('how can this happen?');
                        end;
                    else
                        x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);
                    end;
                end;
            end;
%                 AnyRestrictions_combined*x0_hr(:)
%                 current_options=my_options;
            while ii<NoTrials && min_exit_flag<1
                ii=ii+1;
                if ii==NoTrials && ~any(isnan(q_lb_AnyR(:)))
                    x0_hr = reshape(q_lb_AnyR, nvar, nshocks) ;
                elseif ii>1
                    if min_exit_flag~=0
                        addon = randn(nvar,nshocks);
                        x0_hr = (xfeasible+addon)./repmat(sqrt(sum((xfeasible+addon).^2,1)), nvar,1);
                        fprintf('using random starting point \n');
                    else                            
                        x0_hr = x; 
                        fprintf('continuing from last point after too many iterations \n');
                    end
                end;
                try
                    % it seems much faster when we provide lower and upper bounds
                    sel_vec(FEVD_variables_num(sel_var))=1;
                    [MaxExplained, MinExplained, ~, S]=my_summed_forecast_error_cum_q(biga, [tmp_chol; zeros((lags-1)*nvar, nvar)], nvar, lags, FEVD_variables_num(sel_var), this_horz-1, this_horz-1, Do_Cumulative(FEVD_variables_num(sel_var)));

                    % -----------------------------------------------------------------
                    % FEVD minimization part
                    % -----------------------------------------------------------------

                    obj_fun = @(x) get_summed_forecast_error_mshocks(x, S, shock_idx, nvar, nshocks, Total_FEVD(FEVD_variables_num(sel_var),this_horz));
                    if DO_KNITRO
                        [x, min_val, min_exit_flag, output, min_lagrange_multipliers] = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                        min_val = obj_fun(x); % account for rare Knitro bug in v10.3.2
                        x=reshape(reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                        [~,nl_cons_violation] = my_eq_cons(x);
                        if all(AnyRestrictions_combined*x<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                            switch min_exit_flag
                                case {0, -100, -101, -102, -103}
                                    min_exit_flag=7;
                                case {-400 , -401, -402, -410, -411, -412}
                                    min_exit_flag=0;
                            end;
                        end;
                    else
                        [x, min_val, min_exit_flag, ~, min_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                        if min_exit_flag==0
                            x_num = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                            [x, min_val, min_exit_flag, ~, min_lagrange_multipliers] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                        end;
                    end;
                catch
                    try
                        fprintf(2, 'active set algorithm failed with gradient. trying num gradient \n');
                        [x, min_val, min_exit_flag, ~, min_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                    catch
                        fprintf(2, 'active set algorithm failed with num gradient. try sqp \n');
                        [x, min_val, min_exit_flag, ~, min_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                    end;
                end;
                tmp_q=reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1);
                [~,nl_cons_violation] = my_eq_cons(tmp_q(:));

                if ~all(AnyRestrictions_combined*tmp_q(:)<=b_combined+CommonTol) || (abs(nl_cons_violation)>CommonTol) 
                    min_exit_flag=-9;
                elseif min_exit_flag<0
                    min_exit_flag=100+abs(min_exit_flag);
                end
                
                    % check if an alternative value would do better
                    if DO_ALG2_START && (ii==NoTrials || min_exit_flag>0) && (AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos,shock_idx,1) < min_val - CommonTol || min_exit_flag<0)
                        x0_hr = reshape(AnyR_qext_FEVD_q(sel_var, hh_pos, :, 1,shock_idx), nvar, nshocks);
                        x0_hr = x0_hr./repmat(sqrt(sum(x0_hr.^2,1)), nvar,1);
                        
                        if DO_KNITRO
                            [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                            x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            min_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                            [~,nl_cons_violation] = my_eq_cons(x_alt);
                            if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                switch min_exit_flag_alt
                                    case {0, -100, -101, -102, -103}
                                        min_exit_flag_alt=7;
                                    case {-400 , -401, -402, -410, -411, -412}
                                        min_exit_flag_alt=0;
                                end;
                            end;

                            [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);
                        else
                            try
                                [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                            catch
                                [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                            end;
                            try
                                [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                x=x0_hr(:);
                                min_val_alt  = 1e3;
                                min_exit_flag_alt = 0; 
                            end
                            if min_val_alt > AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos,shock_idx,1) || min_exit_flag_alt<0
                                try
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp  );
                                catch
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                end;
                            end
                            if min_exit_flag_alt==0
                                x=x0_hr(:);
                                try
                                    [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                catch
                                    [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                end;
                                try
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                end;
                            else
                                [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);
                            end;
                        end;
                        [~,eq_const_val]=my_eq_cons(x_alt);
                        [~,eq_const_val_num]=my_eq_cons(x_num);
                        if (min_val_alt < min_val || min_exit_flag<0) && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && min_exit_flag_alt>=0)
                            if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                fprintf('r');
                            end;                        
                            [x, min_val, min_exit_flag,  min_lagrange_multipliers] =deal(x_alt, min_val_alt, 9,  min_lagrange_multipliers_alt );
                        elseif (min_val_num < min_val || min_exit_flag<0) && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && min_exit_flag_alt>=0)
                            if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                fprintf('R');
                            end;                        
                            [x, min_val, ~,  min_lagrange_multipliers] =deal(x_num, min_val_num, 9,  min_lagrange_multipliers_num);
                            min_exit_flag=1;
                        end;
                    end;
                    if hh_pos>1 && DO_CHEB_START && (ii==NoTrials || min_exit_flag>0)
                        x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);

                        if DO_KNITRO
                            [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                            x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            min_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                            [~,nl_cons_violation] = my_eq_cons(x_alt);
                            if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                switch min_exit_flag_alt
                                    case {0, -100, -101, -102, -103}
                                        min_exit_flag_alt=7;
                                    case {-400 , -401, -402, -410, -411, -412}
                                        min_exit_flag_alt=0;
                                end;
                            end;
                            [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);                            
                        else
                            try
                                [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                            catch
                                x=x0_hr(:);
                                min_val_alt  = 1e3;
                                min_exit_flag_alt = 0; 
                            end
                            if min_exit_flag_alt<=0
	                            x=x0_hr(:);
                                try
                                    [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                catch
                                    [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                end;
                                try
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                end;
                            else
                                [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);
                            end;
                        end;
                        [~,eq_const_val]=my_eq_cons(x_alt);
                        [~,eq_const_val_num]=my_eq_cons(x_num);
                        if (min_val_alt < min_val  || min_exit_flag<0) && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && min_exit_flag_alt>=0)
                            if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                fprintf('Y');
                                disp([ min_val_alt - min_val]);
                            end;                        
                            [x, min_val, min_exit_flag,  min_lagrange_multipliers] =deal(x_alt, min_val_alt, 9,  min_lagrange_multipliers_alt );
                        elseif (min_val_num < min_val || min_exit_flag<0)  && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && min_exit_flag_alt>=0)
                            if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                fprintf('X');
                                disp([min_val_alt - min_val]);                          
                            end;                        
                            [x, min_val, ~,  min_lagrange_multipliers] =deal(x_num, min_val_num, 9,  min_lagrange_multipliers_num);
                            min_exit_flag=1;
                        end;
                    end;
                    if DO_RAND_START && (ii==NoTrials || min_exit_flag>0)
                        POSSIBLE_ORDER=perms(1:nshocks);
                        if ii==NoTrials && min_exit_flag==0
                             [~,nl_cons_violation] = my_eq_cons(x);
                             if all(AnyRestrictions_combined*x<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                 min_exit_flag=7;
                             else
                                 min_exit_flag=-7;
                             end;
                        end;
                        JJ=0;
                        while (JJ < 1 + (hh_pos==1)*3 ) 
                            jj=0;
                            JJ = JJ+1;
                        while jj < (ceil(NO_RAND_STARTS)+ceil(NO_RAND_STARTS)+1) || (hh_pos>1 && SOLUTION_POSSIBLE && min_exit_flag<=0)
                            jj=jj+1;
                            if jj>ceil(NO_RAND_STARTS)+1
                                q_idx=randsample(ndraws_q, 1);
                                x0_hr = x_draws(:,:, q_idx);
                                x0_hr = reshape(reshape(x0_hr,nvar, nshocks)./repmat(sqrt(sum(reshape(x0_hr,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            elseif jj>1 || JJ>1 % try random q 
                                current_order=POSSIBLE_ORDER(randsample(factorial(nshocks),1),:);
                                x0_hr=reshape(x,nvar,nshocks);
                                for kk=1:nshocks
                                    new_column(kk:end) = x0_hr(kk:end,current_order(kk))+randn(nvar-kk+1,1);
                                    x0_hr(:,current_order(kk)) = null(x0_hr(:,current_order(1:kk-1))')*new_column(kk:end)/norm(new_column(kk:end));
                                end;
                            else % find q-draw with highest correlation
                                [~,q_idx]=max(corr(reshape(x_draws, nvar*nshocks,ndraws_q), x));
                                x0_hr = x_draws(:,:, q_idx);
                                x0_hr = reshape(reshape(x0_hr,nvar, nshocks)./repmat(sqrt(sum(reshape(x0_hr,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            end;

                            if DO_KNITRO
                                [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                                x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                                min_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                                [~,nl_cons_violation] = my_eq_cons(x_alt);
                                if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                    switch min_exit_flag_alt
                                        case {0, -100, -101, -102, -103}
                                            min_exit_flag_alt=7;
                                        case {-400 , -401, -402, -410, -411, -412}
                                            min_exit_flag_alt=0;
                                    end;
                                end;
                                [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);
                            else
                                try
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                                catch
                                    x=x0_hr(:);
                                    min_val_alt  = 1e3;
                                    min_exit_flag_alt = 0; 
                                end
                                if min_exit_flag_alt<=0
                              		  x=x0_hr(:);

                                    try
                                        [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                    catch
                                        [x_num, min_val_num, min_exit_flag_num, ~, min_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                    end;
                                    try
                                        [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                    catch ME
                                        if isequal(ME.identifier, 'MATLAB:svd:matrixWithNaNInf')
                                            [x_alt, min_val_alt, min_exit_flag_alt,  min_lagrange_multipliers_alt] = deal(x_num, min_val_num, min_exit_flag_num,  min_lagrange_multipliers_num);
                                        end;
                                    end;
                                else
                                    [x_num, min_val_num, min_exit_flag_num, min_lagrange_multipliers_num] = deal(x_alt, min_val_alt, min_exit_flag_alt, min_lagrange_multipliers_alt);
                                end;
                            end;
                            [~,eq_const_val]=my_eq_cons(x_alt);
                            [~,eq_const_val_num]=my_eq_cons(x_num);
                            if (min_val_alt < min_val  || min_exit_flag<0) && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && min_exit_flag_alt>=0)
                                if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                    fprintf('y');
                                    if jj==1
                                        fprintf('1');
                                    elseif jj<ceil(NO_RAND_STARTS)+1
                                        fprintf('q');
                                    end;
                                end;                        
                                [x, min_val, min_exit_flag,  min_lagrange_multipliers] =deal(x_alt, min_val_alt, 9,  min_lagrange_multipliers_alt );
                            elseif (min_val_num < min_val  || min_exit_flag<0) && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && min_exit_flag_alt>=0)
                                if (min_val_alt < min_val - CommonTol) && DO_VERBOSE
                                    fprintf('x');
                                    if jj==1
                                        fprintf('1');
                                    elseif jj<ceil(NO_RAND_STARTS)+1
                                        fprintf('q');
                                    end;
                                end;                        
                                [x, min_val, ~,  min_lagrange_multipliers] =deal(x_num, min_val_num, 9,  min_lagrange_multipliers_num);
                                min_exit_flag=1;
                            end;
                        end;
                        end;
                    end;
                    tmp_q=reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1);
                    q_lb_AnyR = tmp_q(:); % need the entire rotation matrix here, not just the first column
                    AnyR_q(FEVD_variables_num(sel_var),hh_pos,:, 1, shock_idx)=q_lb_AnyR;       
            end;

            % -----------------------------------------------------------------
            % FEVD maximization part
            % -----------------------------------------------------------------
            sel_vec(FEVD_variables_num(sel_var))= -1;
            ii=0;
            max_exit_flag=-1;
            % warm start to optimization problem unless first horizon
            if hh_pos==1 || ~DO_ALG2_START
                % repmat unnecessary in modern Matlab, but cleaner
                x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);
            else
                if ~DO_ALG2_START
                    x0_hr = reshape(AnyR_qext_FEVD_q(FEVD_variables_num(sel_var), hh_pos, :, shock_idx, 2), nvar, nshocks);
                    x0_hr = x0_hr./repmat(sqrt(sum(x0_hr.^2,1)), nvar,1);
                else
                    if ~any(isnan(AnyR_q(FEVD_variables_num(sel_var),hh_pos-1,:, 2, shock_idx))) && DO_CHEB_START
                        x0_hr = AnyR_q(FEVD_variables_num(sel_var),hh_pos-1,:, 2, shock_idx);
                    else
                        x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);
                    end;
                end;
            end;
            while ii<NoTrials && max_exit_flag<1 && min_exit_flag>=0 
                ii=ii+1;
                if ii==NoTrials && ~any(isnan(q_ub_AnyR(:)))
                    x0_hr = reshape(q_ub_AnyR, nvar, nshocks) ;
                elseif ii>1
                    if max_exit_flag~=0
                        addon = randn(nvar,nshocks);
                        x0_hr = (xfeasible+addon)./repmat(sqrt(sum((xfeasible+addon).^2,1)), nvar,1);
                        fprintf('using random starting point \n');
                    else                            
                        x0_hr = x; 
                        fprintf('continuing from last point after too many iterations \n');
                    end
                end;
                try

                    obj_fun = @(x) get_summed_forecast_error_mshocks(x, - S, shock_idx, nvar, nshocks, Total_FEVD(FEVD_variables_num(sel_var),this_horz));
                    %                         tic;
                    if DO_KNITRO
                        [x, max_val, max_exit_flag, ~, max_lagrange_multipliers] = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                        max_val = obj_fun(x); % account for rare Knitro bug in v10.3.2
                        x=reshape(reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                        [~,nl_cons_violation] = my_eq_cons(x);
                        if all(AnyRestrictions_combined*x<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                            switch max_exit_flag
                                case {0, -100, -101, -102, -103}
                                    max_exit_flag=7;
                                case {-400 , -401, -402, -410, -411, -412}
                                    % -410 would be infeasible, but here we
                                    % check feasibility
                                    max_exit_flag=0;
                            end;
                        end;
                    else
                        [x, max_val, max_exit_flag, ~, max_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                        if max_exit_flag==0
                            x_num = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                            [x, max_val, max_exit_flag, ~, max_lagrange_multipliers] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                        end;
                    end;
                catch
                    try
                        fprintf(2, 'active set algorithm failed with gradient. try num differentiation \n');
                        [x, max_val, max_exit_flag, ~, max_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                    catch
                        fprintf(2, 'active set algorithm failed with num gradient. try sqp \n');
                        [x, max_val, max_exit_flag, ~, max_lagrange_multipliers] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                    end;
                end;
                tmp_q=reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1);
                [~,nl_cons_violation] = my_eq_cons(tmp_q(:));

                if ~all(AnyRestrictions_combined*tmp_q(:)<=b_combined+CommonTol) || (abs(nl_cons_violation)>CommonTol) 
                    max_exit_flag=-9;
                elseif max_exit_flag<0
                    max_exit_flag=100+abs(max_exit_flag);
                end 
                    % check if other starting points do better
                    if DO_ALG2_START && (ii==NoTrials || max_exit_flag>0) && (AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos,shock_idx,2) > -max_val + CommonTol || max_exit_flag<0)
                        x0_hr = reshape(AnyR_qext_FEVD_q(sel_var, hh_pos, :, 2,shock_idx), nvar, nshocks);
                        x0_hr = x0_hr./repmat(sqrt(sum(x0_hr.^2,1)), nvar,1);
    
                        if DO_KNITRO
                            [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt]  = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                            x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            max_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                            [~,nl_cons_violation] = my_eq_cons(x_alt);
                            if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                switch max_exit_flag_alt
                                    case {0, -100, -101, -102, -103}
                                        max_exit_flag_alt=7;
                                    case {-400 , -401, -402, -410, -411, -412}
                                        % -410 would be infeasible, but here I
                                        % check feasibility
                                        max_exit_flag_alt=0;
                                end;
                            end;
                            [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                        end
                        if ~DO_KNITRO || (DO_KNITRO && max_exit_flag_alt<=0)
                            x = x0_hr(:);
                            try
                                [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                            catch
                                x = x0_hr(:);
                                max_val_alt   = -1e3;
                                max_exit_flag_alt = 0; 
                            end
                            if -max_val_alt < AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos,shock_idx,2) || max_exit_flag_alt<0
                                try
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp  );
                                catch
                                    [x_alt, min_val_alt, min_exit_flag_alt, ~, min_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                end;
                            end

                            if max_exit_flag_alt==0
                                try
                                    [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                catch
                                    [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                end;
                                try
                                    [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                catch ME
                                    if isequal(ME.identifier, 'MATLAB:svd:matrixWithNaNInf')
                                        [x_alt, max_val_alt, max_exit_flag_alt,  max_lagrange_multipliers_alt] = deal(x_num, max_val_num, max_exit_flag_num,  max_lagrange_multipliers_num);
                                    end;
                                end;
                            else
                                [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                            end;
                        end
                        [~,eq_const_val]=my_eq_cons(x_alt);
                        [~,eq_const_val_num]=my_eq_cons(x_num);
                        if (-max_val_alt > -max_val || max_exit_flag<0)  && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && max_exit_flag_alt>0)
                            if (-max_val_alt > -max_val  +CommonTol) && DO_VERBOSE
                                fprintf('r');
                            end;
                            [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_alt, max_val_alt, 9,  max_lagrange_multipliers_alt );
                        elseif (-max_val_alt > -max_val || max_exit_flag<0) && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && max_exit_flag_alt>=0)
                            if (-max_val_alt > -max_val + CommonTol) && DO_VERBOSE
                                fprintf('R');
                            end;                        
                            [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_num, max_val_num, 9,  max_lagrange_multipliers_num);
                            max_exit_flag=1;
                        end;
                    end;
                    
                    if hh_pos>1 && DO_CHEB_START && (ii==NoTrials || max_exit_flag>0)
                        x0_hr = xfeasible./repmat(sqrt(sum(xfeasible.^2,1)), nvar,1);
                        if DO_KNITRO
                            [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt]  = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                            x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            max_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                            [~,nl_cons_violation] = my_eq_cons(x_alt);
                            if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                switch max_exit_flag_alt
                                    case {0, -100, -101, -102, -103}
                                        max_exit_flag_alt=7;
                                    case {-400 , -401, -402, -410, -411, -412}
                                        % -410 would be infeasible, but here we check feasibility
                                        max_exit_flag_alt=0;
                                end;
                            end;
                            [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                        end;
                        if ~DO_KNITRO || (DO_KNITRO && max_exit_flag_alt<=0)
                            try
                                [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                            catch
                                x = x0_hr(:);
                                max_val_alt   = -1e3;
                                max_exit_flag_alt = 0; 
                            end

                            if max_exit_flag_alt<=0
                                try
                                    [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                catch
                                    [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                end;
                                try
                                    [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                end;
                            else
                                [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                            end;
                        end;
                        [~,eq_const_val]=my_eq_cons(x_alt);
                        [~,eq_const_val_num]=my_eq_cons(x_num);
                        if (-max_val_alt > -max_val || max_exit_flag<0) && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && max_exit_flag_alt>0)
                            if (-max_val_alt > -max_val  +CommonTol) && DO_VERBOSE
                                fprintf('Y');
                            end;
                            [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_alt, max_val_alt, 9,  max_lagrange_multipliers_alt );
                        elseif (-max_val_alt > -max_val || max_exit_flag<0) && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && max_exit_flag_alt>=0)
                            if (-max_val_alt > -max_val + CommonTol ) && DO_VERBOSE
                                fprintf('X');
                            end;                        
                            [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_num, max_val_num, 9,  max_lagrange_multipliers_num);
                            max_exit_flag=1;
                        end;
                    end;
                    if DO_RAND_START  && (ii==NoTrials || max_exit_flag>0)
                        if ii==NoTrials && max_exit_flag==0
                             [~,nl_cons_violation] = my_eq_cons(x);
                             if all(AnyRestrictions_combined*x<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                 max_exit_flag=7;
                             else
                                 max_exit_flag=-7;
                             end;
                        end;
                        POSSIBLE_ORDER=perms(1:nshocks);
                        JJ=0;
                        while (JJ < 1 + (hh_pos==1)*3 ) 
                            jj=0;
                            JJ = JJ+1;
                        while jj < (ceil(NO_RAND_STARTS)+ceil(NO_RAND_STARTS)+1) || (hh_pos>1 && SOLUTION_POSSIBLE && max_exit_flag<=0)
                            jj=jj+1;
                            if jj>ceil(NO_RAND_STARTS)+1
                                q_idx=randsample(ndraws_q, 1);
                                x0_hr = x_draws(:,:, q_idx);
                                x0_hr = reshape(reshape(x0_hr,nvar, nshocks)./repmat(sqrt(sum(reshape(x0_hr,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            elseif jj>1 || JJ>1 % try random q 
                                current_order=POSSIBLE_ORDER(randsample(factorial(nshocks),1),:);
                                x0_hr=reshape(x,nvar,nshocks);
                                for kk=1:nshocks
                                    new_column(kk:end) = x0_hr(kk:end,current_order(kk))+randn(nvar-kk+1,1);
                                    x0_hr(:,current_order(kk)) = null(x0_hr(:,current_order(1:kk-1))')*new_column(kk:end)/norm(new_column(kk:end));
                                end;
                            else % find q-draw with highest correlation
                                [~,q_idx]=max(corr(reshape(x_draws, nvar*nshocks,ndraws_q), x));
                                x0_hr = x_draws(:,:, q_idx);
                                x0_hr = reshape(reshape(x0_hr,nvar, nshocks)./repmat(sqrt(sum(reshape(x0_hr,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                            end;
                            if DO_KNITRO
                                [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt]  = knitromatlab(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,[],knitro_options  );
                                x_alt=reshape(reshape(x_alt,nvar, nshocks)./repmat(sqrt(sum(reshape(x_alt,nvar, nshocks).^2,1)), nvar,1),1,[])'; % renormalize. this can help
                                max_val_alt = obj_fun(x_alt); % account for rare Knitro bug in v10.3.2
                                [~,nl_cons_violation] = my_eq_cons(x_alt);
                                if all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (abs(nl_cons_violation)<CommonTol) 
                                    switch max_exit_flag_alt
                                        case {0, -100, -101, -102, -103}
                                            max_exit_flag_alt=7;
                                        case {-400 , -401, -402, -410, -411, -412}
                                            % -410 would be infeasible, but here we check feasibility
                                            max_exit_flag_alt=0;
                                    end;
                                end;
                                [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                            end;
                            if ~DO_KNITRO || (DO_KNITRO && max_exit_flag_alt<=0)
                                try
                                    [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x0_hr(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options  );
                                catch
                                    x = x0_hr(:);
                                    max_val_alt   = -1e3;
                                    max_exit_flag_alt = 0; 
                                end
                                if max_exit_flag_alt<=0
                                    try
                                        [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num);
                                    catch
                                        [x_num, max_val_num, max_exit_flag_num, ~, max_lagrange_multipliers_num] = fmincon(obj_fun, x, AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp);
                                    end;
                                    try
                                        [x_alt, max_val_alt, max_exit_flag_alt, ~, max_lagrange_multipliers_alt] = fmincon(obj_fun, x_num(:), AnyRestrictions_combined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_many  );
                                    catch ME
                                        if isequal(ME.identifier, 'MATLAB:svd:matrixWithNaNInf')
                                            [x_alt, max_val_alt, max_exit_flag_alt,  max_lagrange_multipliers_alt] = deal(x_num, max_val_num, max_exit_flag_num,  max_lagrange_multipliers_num);
                                        end;
                                    end;
                                else
                                    [x_num, max_val_num, max_exit_flag_num, max_lagrange_multipliers_num] = deal(x_alt, max_val_alt, max_exit_flag_alt, max_lagrange_multipliers_alt);
                                end;
                            end;
                            [~,eq_const_val]=my_eq_cons(x_alt);
                            [~,eq_const_val_num]=my_eq_cons(x_num);
                            if (-max_val_alt > -max_val || max_exit_flag<0)  && all(AnyRestrictions_combined*x_alt<=b_combined+CommonTol) && (eq_const_val<CommonTol && max_exit_flag_alt>0)
                                if (-max_val_alt > -max_val  +CommonTol) && DO_VERBOSE
                                    fprintf('Y');
                                    if jj==1
                                        fprintf('1');
                                    elseif jj<ceil(NO_RAND_STARTS)+1
                                        fprintf('q');
                                    end;
                                end;
                                [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_alt, max_val_alt, 9,  max_lagrange_multipliers_alt );
                            elseif (-max_val_alt > -max_val || max_exit_flag<0) && all(AnyRestrictions_combined*x_num<=b_combined+CommonTol) && (eq_const_val_num<CommonTol && max_exit_flag_alt>=0)
                                if (-max_val_alt > -max_val +CommonTol) && DO_VERBOSE
                                    fprintf('X');
                                    if jj==1
                                        fprintf('1');
                                    elseif jj<ceil(NO_RAND_STARTS)+1
                                        fprintf('q');
                                    end;
                                end;                        
                                [x, max_val, max_exit_flag,  max_lagrange_multipliers] =deal(x_num, max_val_num, 9,  max_lagrange_multipliers_num);
                                max_exit_flag=1;
                            end;
                        end;
                        end;
                    end;
                    tmp_q=reshape(x,nvar, nshocks)./repmat(sqrt(sum(reshape(x,nvar, nshocks).^2,1)), nvar,1);
                    q_ub_AnyR = tmp_q(:);
                    AnyR_q(FEVD_variables_num(sel_var),hh_pos,:, 2, shock_idx)=q_ub_AnyR;                                                    
            end;
            if -max_val<AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos, shock_idx,2)-CommonTol
               fprintf(2, 'check upper bound \n') 
            end
            if min_val>AnyR_FEVD_bound_q(FEVD_variables_num(sel_var),hh_pos, shock_idx,1)+CommonTol
               fprintf(2, 'check lower bound \n') 
            end
            % Store Bounds
            if min_exit_flag>=0 && max_exit_flag>=0
                AnyR_FEVD_bound(sel_var,hh_pos, shock_idx,1) = min_val;
                AnyR_FEVD_bound(sel_var,hh_pos, shock_idx,2) = -max_val;
             
                if this_horz==1 && shock_idx==1 && abs(min_val)~=1e3 && abs(max_val)~=1e3 
                    SOLUTION_POSSIBLE = true;
                elseif this_horz==1 && shock_idx==1 && (abs(min_val)==1e3 || abs(max_val)==1e3)
                    StopComputingRes = true;
                end;

            else
                AnyR_FEVD_bound(sel_var,hh_pos, shock_idx,1:2) = NaN;
                if this_horz==1 && sel_var==1 %% if solutions were feasible at earlier draws, feasible solutions still exist
                    StopComputingRes=true;
                end;
                AnyR_rejected_draws=AnyR_rejected_draws+1;
            end;
        end
    end;
end
