% Now adjusted to allow for FEVD with parameter uncertainty (TD: 3/1/17)
% Now adjusted for problems with fmincon that rarely occurred when a
% generalized inverse failed in the optimization routine (TD: 3/6/17)

%% Compute results at posterior mean
ndraws_q_loop = ndraws_q; % save this option before it is overwritten in the plug-in

Suffix='';

CommonTol = 1e-7;

if exist('knitromatlab.m', 'file')==2
    DO_KNITRO = true;
    knitro_options = optimset('Display','off','Algorithm','active-set', 'TolFun', CommonTol, 'TolX', CommonTol, 'TolCon', CommonTol, 'MaxIter', 1e3, 'MaxFunEvals', 1e6, ...
                                    'UseParallel', false, 'DerivativeCheck', 'off', 'GradObj', 'on', 'GradConstr', 'on');
    knitro_options_num = optimset('Display','off','Algorithm','active-set', 'TolFun', CommonTol, 'TolX', CommonTol, 'TolCon', CommonTol, 'MaxIter', 1e3, 'MaxFunEvals', 1e6, ...
                                    'UseParallel', false, 'DerivativeCheck', 'off', 'GradObj', 'off', 'GradConstr', 'off');
    model_version=model_version*10+3;
else
    DO_KNITRO = false;
end

my_eq_cons = @(z) my_eq_constraint_v2(z, nvar, nshocks);

% Function tolerance 1e-6 makes no discernible difference
my_options = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true, 'CheckGradients', false);
my_options_verb = optimoptions('fmincon','Display','iter-detailed','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);
my_options_many = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e5, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);

my_options_sqp = optimoptions('fmincon','Display','off','Algorithm','sqp', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e5, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);

my_options_num = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', false, 'SpecifyObjectiveGradient', false);

PreFixName = sprintf('%sNBoundsVariable_Model%1.0f_H%d',save_path_bound, model_version,H);
if ParallelParts>1 && ~PLUG_IN_ONLY
   Suffix = [Suffix '_Pl' num2str(WhichParallelPart) '_' num2str(ParallelParts) ];
end
if nstep~=25
   Suffix = [Suffix '_st' num2str(nstep) ];
end

nstep_fevd= length(FEVD_horizons);
nvar_fevd = length(FEVD_variables);
FEVD_variables_num = zeros(nvar_fevd,1);
for nn=1:nvar_fevd 
    FEVD_variables_num(nn,1) = find(strcmp(FEVD_variables{nn}, varname));
end;

AnyR_q = zeros(nvar_fevd, nstep_fevd, nvar*nshocks, 2, nshocks);
SR_q = zeros(nvar_fevd, nstep_fevd, nvar*nshocks, nshocks, 2);
HR_q = zeros(nvar_fevd, nstep_fevd, nvar*nshocks, nshocks, 2);

% Now, there can also be positive Lagrange multipliers on the restrictions 
% on the other shock(s)
res_total_count = 0;
for ss=1:nshocks
    for hh=H_min:H_max
        h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
        res_total_count = res_total_count + sum(h_idx);
    end;
    res_total_count = res_total_count + nres_slope(ss);
end;


Max_FEVD_reps=500;
if ispc
    FEVD_distribution= matfile( [regexprep(PreFixName, 'Graphs', 'Results') '_FEVD.mat'],'Writable',true);
end;

FEVD_distribution.SR_FEVD_bound = nan(nvar_fevd, nstep_fevd, nshocks, min(CurrentDraws,Max_FEVD_reps), 2);
FEVD_distribution.HR_FEVD_bound = nan(nvar_fevd, nstep_fevd, nshocks, min(CurrentDraws,Max_FEVD_reps), 2);   

FEVD_distribution.SR_FEVD_bound_q = nan(nvar_fevd, nstep_fevd, nshocks, min(CurrentDraws,Max_FEVD_reps), ndraws_q_loop);
FEVD_distribution.HR_FEVD_bound_q = nan(nvar_fevd, nstep_fevd, nshocks, min(CurrentDraws,Max_FEVD_reps), ndraws_q_loop);

AnyR_qext_FEVD_q = nan(nvar_fevd, nstep_fevd, nvar*nshocks, 2, nshocks);
AnyR_FEVD_bound = nan(nvar_fevd, nstep_fevd, nshocks, 2);

DO_VERBOSE_CORRECTION=false;

%%
ndraws_q        = ndraws_q_loop ; % restore to value before plug-in
SRestrictionsA = cell(nshocks,1);
HRestrictionsA = cell(nshocks,1);
AnyR_rejected_draws = 0;

ndraws_q_loop        = ndraws_q;
ndraws_q_mean        = min(1e5, ndraws_q*10);

% for every horizon, for computational simplicity
AnyR_FEVD_q = nan(nvar, nstep, nshocks, ndraws_q);
% only for the desired horizons
AnyR_FEVD_bound_q = nan(nvar, nstep_fevd, nshocks, 2);

% ndraws=0;
draw_start = (WhichParallelPart-1)*CurrentDraws ; 

draw_i=(WhichParallelPart-1)*CurrentDraws+(WhichParallelPart>1);
if (draw_i == 0)
    DO_VERBOSE=true;
else
    DO_VERBOSE=false;
end;
tic;

if PLUG_IN_ONLY
    draw_i       = 0;
    draw_start   = 0;
    CurrentDraws = 0;
end
if ~exist('DO_SLOPE_W_SR', 'var')
    DO_SLOPE_W_SR=true;
end;

% loop over reduced form draws
while (draw_i <= draw_start+CurrentDraws)
    CURRENTLY_HR = false;
    
    biga = zeros(nvar*lags,nvar*lags);
    for j = 1:lags-1
        biga(j*nvar+1:nvar*(j+1),nvar*(j-1)+1:j*nvar) = eye(nvar);
    end
    if draw_i > 0
        for ii = 1:lags
            biga(1:nvar,(ii-1)*nvar+1:ii*nvar) = squeeze(By(:,:,ii,draw_i));
        end
        tmp_chol = chol(squeeze(omega(:,:,draw_i)),'lower');
    else
        for ii = 1:lags
            biga(1:nvar,(ii-1)*nvar+1:ii*nvar) = squeeze(mean(By(:,:,ii,:),4));
        end
        tmp_chol = chol(squeeze(mean(omega(:,:,:),3)),'lower');
    end;
    
    res_total_count = 0;
    res_shock_count = cell(nshocks);
    
    % loop over shocks
    for ss=1:nshocks
        res_shock_count{ss} = 0;
        for hh=H_min:H_max
            h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
            res_shock_count{ss} = res_shock_count{ss} + sum(h_idx);
            res_total_count = res_total_count + sum(h_idx);
        end;
        res_shock_count{ss} = res_shock_count{ss} + nres_slope(ss);
        res_total_count = res_total_count + nres_slope(ss);
    end;
    SRestrictions_combined = zeros(res_total_count,nvar*nshocks);
    b_combined = zeros(res_total_count,1);
    ones_combined = ones(nvar*nshocks,1);
    res_total_count = 0;
    for ss=1:nshocks
        SRestrictionsA{ss,1} =zeros(res_shock_count{ss},nvar);
        res_running_count = 0;
        for hh=H_min:H_max
            h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
            if hh>1
                SRestrictionsA{ss,1}(res_running_count+(1:sum(h_idx)),:)=  - (SS{ss,1}(h_idx,:)*EE{ss,1}*...
                    ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,hh, 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,hh, 0)) );
            else
                SRestrictionsA{ss,1}( 1:sum(h_idx) ,:)= - (SS{ss,1}(h_idx,:)*EE{ss,1}*tmp_chol);
            end;
            res_running_count = res_running_count + sum(h_idx);
        end;
        if DO_SLOPE_W_SR %otherwise just leave at zero
            for nn=1:nres_slope(ss)
                SRestrictionsA{ss,1}(res_running_count+nn,:) = ...
                     -SS_slope{ss,1}(nn,:)*EE_slope{ss,1}*...
                           ( ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 0)) ...
                           - ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 0)) );
            end;
        end;
        res_running_count = res_running_count + nres_slope(ss);
        SRestrictions_combined( res_total_count  + (1:res_running_count ), (ss-1)*nvar+(1:nvar) ) = SRestrictionsA{ss,1};
        res_total_count = res_total_count + res_running_count;
    end;
    
    AnyRestrictions_combined = SRestrictions_combined;
    AnyRestrictionsA = SRestrictionsA;
    
    %call innermost loop: Alg 2 and Alg 1 given parameters
    if ~DO_KNITRO
        error('currently only set up for Knitro');
    else
        IdentificationGivenReducedForm_Inner_Knitro_FEVD;
    end;
    
    if ~StopComputingRes
        % for posterior mean analysis, save objects elsewhere
        if draw_i==0
            Mean_SR_FEVD_bound =  AnyR_FEVD_bound;
            Mean_SIGN_FEVD_SR =  AnyR_FEVD_q;
            SR_q = AnyR_q;            
        else
            FEVD_distribution.SR_FEVD_bound(:,:,:,draw_i- draw_start,:) = reshape(AnyR_FEVD_bound, [nvar_fevd,nstep_fevd, nshocks, 1,2]);
            FEVD_distribution.SR_FEVD_bound_q(:,:,:,draw_i- draw_start,:) = reshape(AnyR_FEVD_q(FEVD_variables_num,FEVD_horizons,:,:), [nvar_fevd,nstep_fevd, nshocks, 1, ndraws_q]);
            SR_q = AnyR_q;            
        end;
    end;
    if mod(draw_i,1)==0;fprintf('Identification up to SR part of draw:\t%5.0f (%5.2f minutes)\n',draw_i,toc/60);end
    
    if ll~=0 || (~DO_SLOPE_W_SR && max(nres_slope)>0)
        % =====================================================================
        % Nummerical Bounds via Heterogeneity Restrictions
        % =====================================================================
        HRestrictions_combined = zeros(size(SRestrictions_combined ));
        ones_combined = ones(nvar*nshocks,1);
        res_total_count = 0;
        for ss=1:nshocks
            HRestrictionsA{ss,1} =zeros(res_shock_count{ss},nvar);
            res_running_count = 0;
            for hh=H_min:H_max
                h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
                if hh>1
                    HRestrictionsA{ss,1}(res_running_count+(1:sum(h_idx)),:)=  - (SS{ss,1}(h_idx,:)*(EE{ss,1}-LL_past_impact{ss,1})*...
                        ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,hh, 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,hh, 0)) );
                else
                    HRestrictionsA{ss,1}( 1:sum(h_idx) ,:)= - (SS{ss,1}(h_idx,:)*(EE{ss,1}-LL_past_impact{ss,1})*tmp_chol);
                end;
                res_running_count = res_running_count + sum(h_idx);
            end;
            for nn=1:nres_slope(ss)
                HRestrictionsA{ss,1}(res_running_count+nn,:) = ...
                     -SS_slope{ss,1}(nn,:)*EE_slope{ss,1}*...
                           ( ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 0)) ...
                           - ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 0)) );
            end;
            res_running_count = res_running_count + nres_slope(ss);
            HRestrictions_combined( res_total_count  + (1:res_running_count ), (ss-1)*nvar+(1:nvar) ) = HRestrictionsA{ss,1};
            res_total_count = res_total_count + res_running_count;
        end;
        
        AnyRestrictions_combined = HRestrictions_combined;
        AnyRestrictionsA = HRestrictionsA;

        CURRENTLY_HR = true;
        %call innermost loop: Alg 2 and Alg 1 given parameters
        if ~DO_KNITRO
            error('currently only set up for Knitro');
        else
            IdentificationGivenReducedForm_Inner_Knitro_FEVD;
        end;
            
        if ~StopComputingRes
            % for posterior mean analysis, save objects elsewhere
            if draw_i==0
                Mean_HR_FEVD_bound =  AnyR_FEVD_bound;
                Mean_SIGN_FEVD_HR =  AnyR_FEVD_q;
                HR_q = AnyR_q;            
            else
                FEVD_distribution.HR_FEVD_bound(:,:,:,draw_i- draw_start,:) = reshape(AnyR_FEVD_bound, [nvar_fevd,nstep_fevd, nshocks, 1,2]);
                FEVD_distribution.HR_FEVD_bound_q(:,:,:,draw_i- draw_start,:) = reshape(AnyR_FEVD_q(FEVD_variables_num,FEVD_horizons,:,:), [nvar_fevd,nstep_fevd, nshocks, 1, ndraws_q]);
                HR_q = AnyR_q;            
            end;
        end;
    end;
    
    if mod(draw_i,1)==0;fprintf('Identification for draw:\t%5.0f (%5.2f minutes)\n',draw_i,toc/60);end

    if (draw_i == 0) && DO_SAVE
        save([PreFixName 'PlugIn' Suffix]);
        DO_VERBOSE=false;  
    end;

    draw_i=draw_i+1;
end



