nlambdas = length(HR_MULTIPLE);
if nlambdas >1
   fprintf(2, 'Multiple lambdas -- only IRFs are saved \n');
end

%% Compute results at posterior mean
ndraws_q_loop = ndraws_q; % save this option before it is overwritten in the plug-in
if nlambdas >1
    Suffix = num2str(nlambdas,'_NoLam%1.0f');
else
    Suffix='';
end
CommonTol = 1e-7;

if exist('knitromatlab.m', 'file')==2
    DO_KNITRO = true;
    knitro_options = optimset('Display','off','Algorithm','active-set', 'TolFun', CommonTol, 'TolX', CommonTol, 'TolCon', CommonTol, 'MaxIter', 1e3, 'MaxFunEvals', 1e6, ...
                                    'UseParallel', false, 'DerivativeCheck', 'off', 'GradObj', 'on', 'GradConstr', 'on');
    knitro_options_num = optimset('Display','off','Algorithm','active-set', 'TolFun', CommonTol, 'TolX', CommonTol, 'TolCon', CommonTol, 'MaxIter', 1e3, 'MaxFunEvals', 1e6, ...
                                    'UseParallel', false, 'DerivativeCheck', 'off', 'GradObj', 'off', 'GradConstr', 'off');
    model_version=model_version*10+3;
else
    DO_KNITRO = false;
end

my_eq_cons = @(z) my_eq_constraint_v2(z, nvar, nshocks);

% Function tolerance 1e-6 makes no discernible difference
my_options = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true, 'CheckGradients', false);
my_options_verb = optimoptions('fmincon','Display','iter-detailed','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);
my_options_many = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e5, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);

my_options_sqp = optimoptions('fmincon','Display','off','Algorithm','sqp', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e5, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', true, 'SpecifyObjectiveGradient', true);

my_options_num = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
    'UseParallel', false, 'SpecifyConstraintGradient', false, 'SpecifyObjectiveGradient', false);

PreFixName = sprintf('%sNBoundsVariable_Model%1.0f_H%d',save_path_bound, model_version,H);
if ParallelParts>1 && ~PLUG_IN_ONLY
   Suffix = [Suffix '_Pl' num2str(WhichParallelPart) '_' num2str(ParallelParts) ];
end
if nstep~=25
   Suffix = [Suffix '_st' num2str(nstep) ];
end

AnyR_q = zeros(nvar, nstep, nvar*nshocks, nshocks, 2);
if nlambdas<=1
    SR_q = zeros(nvar, nstep, nvar*nshocks, nshocks, 2);
    HR_q = zeros(nvar, nstep, nvar*nshocks, nshocks, 2);
    
    if ispc
        LagrangeMultipliers= matfile( [regexprep(PreFixName, 'Graphs', 'Results') '_LagrangeM' Suffix '.mat'],'Writable',true);
    end
end

% With multiple shocks there can also be positive Lagrange multipliers on the restrictions on the other shock(s)
res_total_count = 0;
for ss=1:nshocks
    for hh=H_min:H_max
        h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
        res_total_count = res_total_count + sum(h_idx);
    end
    res_total_count = res_total_count + nres_slope(ss);
end

LagrangeMultipliers.Mean_SR = nan(nvar,res_total_count,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max        
LagrangeMultipliers.Mean_HR = nan(nvar,res_total_count,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max      
LagrangeMultipliers.Mean_SR_eqnonlin = nan(nvar,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max        
LagrangeMultipliers.Mean_HR_eqnonlin = nan(nvar,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max        

if ~PLUG_IN_ONLY
    LagrangeMultipliers.SR = nan(nvar,res_total_count,nstep, nshocks, CurrentDraws,2); % 5th dimension: = 1: min, 2: max        
    LagrangeMultipliers.HR = nan(nvar,res_total_count,nstep, nshocks, CurrentDraws,2); % 5th dimension: = 1: min, 2: max        
    LagrangeMultipliers.SR_eqnonlin = nan(nvar,nstep, nshocks, CurrentDraws,2); % 5th dimension: = 1: min, 2: max        
    LagrangeMultipliers.HR_eqnonlin = nan(nvar,nstep, nshocks, CurrentDraws,2); % 5th dimension: = 1: min, 2: max        
end

LM_Mean_AnyR = nan(nvar,res_total_count,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max        
LM_Mean_AnyR_eqnonlin = nan(nvar,nstep, nshocks, 2); % 4th dimension: = 1: min, 2: max        

if DO_FEVD~=0
    error('FEVD with multiple shocks not set up, yet');
end
if DO_FEVD==2
   Max_FEVD_reps=100;
   if ispc
        FEVD_distribution= matfile( [regexprep(PreFixName, 'Graphs', 'NewRawResults') '_FEVD.mat'],'Writable',true);
   end
   
   FEVD_distribution.SR_FEVD_bound = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), 4);
   FEVD_distribution.HR_FEVD_bound = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), 4);   
   
   FEVD_distribution.SR_FEVD_q = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), ndraws_q_loop);
   FEVD_distribution.HR_FEVD_q = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), ndraws_q_loop);

   FEVD_distribution.SR_FEVD_bound_q = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), 1+nlambdas);
   FEVD_distribution.HR_FEVD_bound_q = nan(nvar, nstep, nshocks, min(CurrentDraws,Max_FEVD_reps), 1+nlambdas);
   
   SR_qext_FEVD_q = nan(nvar, nstep, nvar*nshocks, 2, nshocks);
   HR_qext_FEVD_q = nan(nvar, nstep, nvar*nshocks, 2, nshocks);

   AnyR_qext_FEVD_q = nan(nvar, nstep, nvar*nshocks, 2, nshocks);
   AnyR_FEVD_bound = nan(nvar, nstep, nshocks, 4);
end
AnyR_qext_IRF_q = nan(nvar, nstep, nvar*nshocks, nshocks, 2);
AnyR_IRF_bound_q = nan(nvar, nstep, nshocks, 2);

AnyRimpf_bound = NaN(nvar,nstep,nshocks,2); % 4th dimension: = 1: min, 2: max

DO_VERBOSE_CORRECTION=false;

%%
ndraws_q       = ndraws_q_loop ; % restore to value before plug-in
SRestrictionsA = cell(nshocks,1);
HRestrictionsA = cell(nshocks,1);
AnyR_rejected_draws = 0;

ndraws_q_loop        = ndraws_q;
ndraws_q_mean        = min(1e5, ndraws_q*10);

Mean_SRimpf_bound    = NaN(nvar,nstep,nshocks, 2); % 3rd dimension: = 1: min, 2: max
Mean_HRimpf_bound    = NaN(nvar,nstep,nshocks, 2, nlambdas); % 3rd dimension: = 1: min, 2: max
SRimpf_bound = NaN(nvar,nstep,nshocks,CurrentDraws, 2); % 4th dimension: = 1: min, 2: max(:,:,:,draw_i- draw_start,:)
HRimpf_bound = NaN(nvar,nstep,nshocks,CurrentDraws, 2, nlambdas); % 4th dimension: = 1: min, 2: max(:,:,:,draw_i- draw_start,:)

Mean_SIGN_IRFS_SR    = NaN(nvar,nstep,nshocks,ndraws_q); % 
Mean_SIGN_IRFS_HR    = NaN(nvar,nstep,nshocks,ndraws_q, nlambdas); % 
SIGN_IRFS_SR    = NaN(nvar,nstep,nshocks, CurrentDraws,ndraws_q); % 
SIGN_IRFS_HR    = NaN(nvar,nstep,nshocks,CurrentDraws, ndraws_q, nlambdas); % 

draw_start = (WhichParallelPart-1)*CurrentDraws ; 

draw_i=(WhichParallelPart-1)*CurrentDraws+(WhichParallelPart>1);
if (draw_i == 0)
    DO_VERBOSE=true;
else
    DO_VERBOSE=false;
end
tic;

if PLUG_IN_ONLY
    draw_i       = 0;
    draw_start   = 0;
    CurrentDraws = 0;
end
if ~exist('DO_SLOPE_W_SR', 'var')
    DO_SLOPE_W_SR=true;
end

while (draw_i <= draw_start+CurrentDraws)
    CURRENTLY_HR = false;
    
    biga = zeros(nvar*lags,nvar*lags);
    for j = 1:lags-1
        biga(j*nvar+1:nvar*(j+1),nvar*(j-1)+1:j*nvar) = eye(nvar);
    end
    if draw_i > 0
        for ii = 1:lags
            biga(1:nvar,(ii-1)*nvar+1:ii*nvar) = squeeze(By(:,:,ii,draw_i));
        end
        tmp_chol = chol(squeeze(omega(:,:,draw_i)),'lower');
    else
        for ii = 1:lags
            biga(1:nvar,(ii-1)*nvar+1:ii*nvar) = squeeze(mean(By(:,:,ii,:),4));
        end
        tmp_chol = chol(squeeze(mean(omega(:,:,:),3)),'lower');
    end
    
    res_total_count = 0;
    res_shock_count = cell(nshocks);
    for ss=1:nshocks
        res_shock_count{ss} = 0;
        for hh=H_min:H_max
            h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
            res_shock_count{ss} = res_shock_count{ss} + sum(h_idx);
            res_total_count = res_total_count + sum(h_idx);
        end
        res_shock_count{ss} = res_shock_count{ss} + nres_slope(ss);
        res_total_count = res_total_count + nres_slope(ss);
    end
    SRestrictions_combined = zeros(res_total_count,nvar*nshocks);
    b_combined = zeros(res_total_count,1);
    ones_combined = ones(nvar*nshocks,1);
    res_total_count = 0;
    for ss=1:nshocks
        SRestrictionsA{ss,1} =zeros(res_shock_count{ss},nvar);
        res_running_count = 0;
        for hh=H_min:H_max
            h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
            if hh>1
                SRestrictionsA{ss,1}(res_running_count+(1:sum(h_idx)),:)=  - (SS{ss,1}(h_idx,:)*EE{ss,1}*...
                    ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,hh, 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,hh, 0)) );
            else
                SRestrictionsA{ss,1}( 1:sum(h_idx) ,:)= - (SS{ss,1}(h_idx,:)*EE{ss,1}*tmp_chol);
            end
            res_running_count = res_running_count + sum(h_idx);
        end
        if DO_SLOPE_W_SR %otherwise just leave at zero
            for nn=1:nres_slope(ss)
                SRestrictionsA{ss,1}(res_running_count+nn,:) = ...
                     -SS_slope{ss,1}(nn,:)*EE_slope{ss,1}*...
                           ( ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 0)) ...
                           - ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 0)) );
            end
        end
        res_running_count = res_running_count + nres_slope(ss);
        SRestrictions_combined( res_total_count  + (1:res_running_count ), (ss-1)*nvar+(1:nvar) ) = SRestrictionsA{ss,1};
        res_total_count = res_total_count + res_running_count;
    end
    
    AnyRestrictions_combined = SRestrictions_combined;
    AnyRestrictionsA = SRestrictionsA;
    
    %call innermost loop: Alg 2 and Alg 1 given parameters
    if ~DO_KNITRO
        IdentificationGivenReducedForm_Inner;
    else
        IdentificationGivenReducedForm_Inner_Knitro;
    end
    
    if ~StopComputingRes
        % for posterior mean analysis, save objects elsewhere
        if draw_i==0
            LagrangeMultipliers.Mean_SR_eqnonlin = LM_Mean_AnyR_eqnonlin;
            LagrangeMultipliers.Mean_SR = LM_Mean_AnyR;
            Mean_SRimpf_bound =  AnyRimpf_bound;
            Mean_SIGN_IRFS_SR =  SIGN_IRFS_AnyR;
            SR_q = AnyR_q;            
        else
            if nlambdas <= 1
                LagrangeMultipliers.SR_eqnonlin(:,:,:, draw_i- draw_start,:)=reshape(LM_Mean_AnyR_eqnonlin, [nvar,nstep, nshocks, 1,2]);
                LagrangeMultipliers.SR(:,:,:,:, draw_i- draw_start,:)=reshape(LM_Mean_AnyR, [nvar,res_total_count,nstep, nshocks, 1,2]);
            end
            SRimpf_bound(:,:,:,draw_i- draw_start,:) = reshape(AnyRimpf_bound, [nvar,nstep, nshocks, 1,2]);
            for ss=1:nshocks
                SIGN_IRFS_SR(:,:,ss,draw_i- draw_start,:) = reshape(SIGN_IRFS_AnyR(:,:,ss,:), [nvar,nstep, 1, 1,ndraws_q ]);
            end
        end
    end
    if mod(draw_i,1)==0;fprintf('Identification up to SR part of draw:\t%5.0f (%5.2f minutes)\n',draw_i,toc/60);end
    
    if ll~=0 || (~DO_SLOPE_W_SR && max(nres_slope)>0)
        % =====================================================================
        % Nummerical Bounds via Heterogeneity Restrictions
        % =====================================================================
        for nn_lam=1:nlambdas
            for ss=1:nshocks
                LL_past_impact{ss,1} = LL_past_impact_original{ss,1}*HR_MULTIPLE(nn_lam);
                LL{ss,1} = LL_original{ss,1}*HR_MULTIPLE(nn_lam);
            end
            
            HRestrictions_combined = zeros(size(SRestrictions_combined ));
            ones_combined = ones(nvar*nshocks,1);
            res_total_count = 0;
            for ss=1:nshocks
                HRestrictionsA{ss,1} =zeros(res_shock_count{ss},nvar);
                res_running_count = 0;
                for hh=H_min:H_max
                    h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
                    if hh>1
                        HRestrictionsA{ss,1}(res_running_count+(1:sum(h_idx)),:)=  - (SS{ss,1}(h_idx,:)*(EE{ss,1}-LL_past_impact{ss,1})*...
                            ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,hh, 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,hh, 0)) );
                    else
                        HRestrictionsA{ss,1}( 1:sum(h_idx) ,:)= - (SS{ss,1}(h_idx,:)*(EE{ss,1}-LL{ss,1})*tmp_chol); % replace LL_past_impact with LL
                    end
                    res_running_count = res_running_count + sum(h_idx);
                end
                for nn=1:nres_slope(ss)
                    HRestrictionsA{ss,1}(res_running_count+nn,:) = ...
                         -SS_slope{ss,1}(nn,:)*EE_slope{ss,1}*...
                               ( ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,2), 0)) ...
                               - ( Do_CumulativeMatrix.*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 1)  + (1-Do_CumulativeMatrix).*setup_irf(biga,tmp_chol,J,H_bounds_slope{ss}(nn,1), 0)) );
                end
                res_running_count = res_running_count + nres_slope(ss);
                HRestrictions_combined( res_total_count  + (1:res_running_count ), (ss-1)*nvar+(1:nvar) ) = HRestrictionsA{ss,1};
                res_total_count = res_total_count + res_running_count;
            end

            AnyRestrictions_combined = HRestrictions_combined;
            AnyRestrictionsA = HRestrictionsA;

            CURRENTLY_HR = true;
            %call innermost loop: Alg 2 and Alg 1 given parameters
            if ~DO_KNITRO
                IdentificationGivenReducedForm_Inner;
            else
                IdentificationGivenReducedForm_Inner_Knitro;
            end
        
            if ~StopComputingRes
                if draw_i==0
                    LagrangeMultipliers.Mean_HR_eqnonlin = LM_Mean_AnyR_eqnonlin;
                    LagrangeMultipliers.Mean_HR = LM_Mean_AnyR;
                    Mean_HRimpf_bound(:,:,:,:,nn_lam) =  AnyRimpf_bound;
                    Mean_SIGN_IRFS_HR(:,:,:,:,nn_lam) =  reshape(SIGN_IRFS_AnyR, [nvar, nstep, nshocks, ndraws_q, 1]);
                    HR_q = AnyR_q;

                    if usejava('jvm') && nstep>1
                        close all
                        for WhichShock=1:nshocks
                            for ii=1:nvar
                                figure(ii+(WhichShock-1)*nvar);
                                plot(0 :nstep-1, squeeze(Mean_HRimpf_bound(ii,:,WhichShock,:,nn_lam)), '--b', 'LineWidth', 2)
                                hold all
                                plot(0 :nstep-1, prctile(squeeze(Mean_SIGN_IRFS_HR(ii,:,WhichShock,:,nn_lam)),[0 1 16  68 99 100],2), '-b', 'LineWidth', 1)
                                plot(0 :nstep-1, squeeze(Mean_SRimpf_bound(ii,:,WhichShock,:)), '-.r', 'LineWidth', 2)
                                hold all
                                plot(0 :nstep-1, prctile(squeeze(Mean_SIGN_IRFS_SR(ii,:,WhichShock,:)),[0 1 16  68 99 100],2), ':r', 'LineWidth', 1)
                                title([varname{ii} ' - shock ' num2str(WhichShock)])
                                grid on
                                axis([0 nstep-1 ylim])
                                if ndraws_q_mean>=5e3 && DO_ALG2_START && DO_CHEB_START && DO_RAND_START
                                    if H~=3
                                        print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_H' num2str(H) '_mod' num2str(model_version) '_lam' num2str(HR_MULTIPLE(nn_lam)) '.eps'])
                                    else
                                        print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_mod' num2str(model_version) '_lam' num2str(HR_MULTIPLE(nn_lam)) '.eps'])
                                    end
                                else
                                    suffix='';
                                    if ~DO_RAND_START 
                                        suffix=[suffix '_no_rnd'];
                                    end
                                    if ~DO_CHEB_START 
                                        suffix=[suffix '_no_cheb'];
                                    end
                                    if ~DO_ALG2_START 
                                        suffix=[suffix '_no_alg2'];
                                    end
                                    if DO_RAND_START
                                        print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_H' num2str(H) suffix '_mod' num2str(model_version) '_lam' num2str(HR_MULTIPLE(nn_lam)) '.eps'])
                                    end
                                end
                            end
                        end
                    end
                    DO_VERBOSE=false;            
                else
                    if nlambdas<=1
                        LagrangeMultipliers.HR_eqnonlin(:,:,:, draw_i - draw_start,:)=reshape(LM_Mean_AnyR_eqnonlin, [nvar,nstep, nshocks, 1,2]);
                        LagrangeMultipliers.HR(:,:,:,:, draw_i- draw_start,:)=reshape(LM_Mean_AnyR, [nvar,res_total_count,nstep, nshocks, 1,2]);
                    end
                    HRimpf_bound(:,:,:,draw_i- draw_start,:, nn_lam) = reshape(AnyRimpf_bound, [nvar,nstep, nshocks, 1,2]);
                    for ss=1:nshocks
                        SIGN_IRFS_HR(:,:,ss,draw_i- draw_start, :,nn_lam) = reshape(SIGN_IRFS_AnyR(:,:,ss,:), [nvar,nstep, ndraws_q, 1,1 ]);
                    end
                end
            end
        end
    else
        LagrangeMultipliers.Mean_HR_eqnonlin = LM_Mean_AnyR_eqnonlin;
        LagrangeMultipliers.Mean_HR = LM_Mean_AnyR;
        Mean_HRimpf_bound =  AnyRimpf_bound;
        Mean_SIGN_IRFS_HR =  SIGN_IRFS_AnyR;

        if usejava('jvm') && nstep>1
            close all
            for WhichShock=1:nshocks
                for ii=1:nvar
                    figure(ii+(WhichShock-1)*nvar);
                    plot(0 :nstep-1, squeeze(Mean_SRimpf_bound(ii,:,WhichShock,:)), '-.r', 'LineWidth', 2)
                    hold all
                    plot(0 :nstep-1, prctile(squeeze(Mean_SIGN_IRFS_SR(ii,:,WhichShock,:)),[0 1 16  68 99 100],2), ':r', 'LineWidth', 1)
                    title([varname{ii} ' - shock ' num2str(WhichShock)])
                    grid on
                    axis([0 nstep-1 ylim])
                    if ndraws_q_mean>=5e3 && DO_ALG2_START && DO_CHEB_START && DO_RAND_START
                        if H~=3
                            print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_H' num2str(H) '_mod' num2str(model_version) '.eps'])
                        else
                            print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_mod' num2str(model_version) '.eps'])
                        end
                    else
                        suffix='';
                        if ~DO_RAND_START 
                            suffix=[suffix '_no_rnd'];
                        end
                        if ~DO_CHEB_START 
                            suffix=[suffix '_no_cheb'];
                        end
                        if ~DO_ALG2_START 
                            suffix=[suffix '_no_alg2'];
                        end
                        if DO_RAND_START
                            print('-depsc', ['TmpGraphs\Var' num2str(ii) '_shock' num2str(WhichShock) '_H' num2str(H) suffix '_mod' num2str(model_version) '_lam' num2str(HR_MULTIPLE(nn_lam)) '.eps'])
                        end
                    end
                end
            end
        end        
    end
    if mod(draw_i,1)==0;fprintf('Identification for draw:\t%5.0f (%5.2f minutes)\n',draw_i,toc/60);end

    if (draw_i == 0) && DO_SAVE
        save([PreFixName 'PlugIn' Suffix], '-v7.3');
        DO_VERBOSE=false;  
    end

    draw_i=draw_i+1;
end

