% Code to extent our baseline analysis of TFP news shocks to a second
% surprise TFP shock
% Currently works for case 11 only

%% Code to compare Sign Restriction to Heterogeneity Restriciton in a VAR
% -------------------------------------------------------------------------
clear('all'); close('all'); 

% addpath('C:\Program Files\Artelys\Knitro 10.3.2\knitromatlab');
%rmpath('C:\Program Files\Artelys\Knitro 10.3.2\knitromatlab');
addpath('C:\Program Files\Artelys\Knitro 12.0.0\knitromatlab');
%rmpath('C:\Program Files\Artelys\Knitro 11.0.0\knitromatlab');

main_path = pwd;
load_path = '';
cd(main_path)
PreFixName='News';
save_path_bound='Graphs/';

% Specifications:
% -------------------------------------------------------------------------
DO_SAVE = true;
DO_PLOT = 0;
DO_FEVD = 0; % >0: Do FEVD at posterior mean. =2: Also do for each draw

DO_SLOPE_W_SR = false;

START_1983 = true ;

% "Tuning" parameters for Algorithm 1
DO_ALG2_START = true; % check if best draw yields a better outcome in fmincon
DO_CHEB_START = true; % check if Chebychev yields a better outcome in fmincon. If false, don't do warm starts using last horizon's q and use Chebychev there
DO_RAND_START =  true; % check if random perturbation yields a better outcome in fmincon
NO_RAND_STARTS = 5; % number of random starting points
ndraws_q    = 5000; % number of draws from marginal posterior over q
ndraws      = 2500; % number of reduced form draws
nstep       = 25;               % number of steps for impulse responses

GibbsStep   = 10;

WhichSeed = 42; % 
rng(WhichSeed)

% Split sample into parts? 
ParallelParts = 10; % this many parallel parts
WhichParallelPart = 8; % here, do computations for this slice 
CurrentDraws = ceil(ndraws/ParallelParts);
%% PART ONE: LOADING DATA AND SPECIFICATIONS
% -------------------------------------------------------------------------


% Grid for strength of heterogeneity restrictions
HR_MULTIPLE = 1/3:1/9:3;
model_version   = 11;  % 0, 2: no micro data. 11, 21: used in paper. 31: Like 11, but no restriction on hours. %41 5 FF + C. % 51 10 FF + C % 91: Book to Market
%     case {1,11,31} % data with R&D heterogeneity
% 99: identify TFP surprise and TFP news with added slope restrictions
% {74, 7401, 7402, 7410, 7411, 7412} %small macro only VAR
% 749 same, just TFP news shock
% 748 same, just TFP surprise shock
% 747 same, just one monetary policy shock
% 666: Reversed HR
% {1,11,31} % data with R&D heterogeneity, 5 industries
% 87 %Small VAR, GS2

H           = 5 ; % Horizon for restrictions to hold
TFP_Lambda=0;
% TFP_Lambda=1; % Output rises more than it would just based on TFP --
% TFP_Lambda=10; % used in paper
% TFP_Lambda=20; % used for future reference with model versions 0 and 2.

file_name   = 'Data/TFP_Stock_Data.csv'; 


%% set up data and restrictions
my_model_setup;

%% set up soft zero restrictions


if TFP_Lambda~=0
   error('Soft zero on TFP currently not incorporated');
   [SS_TFP, EE_TFP, LL_TFP, Do_Cumulative_R_TFP]=my_EnglishToMatrices({'Output>TFP', 1},IncludeSeriesNames,[]);
   LL_TFP = LL_TFP*TFP_Lambda;
   RestrictionLabels=[RestrictionLabels;{'Output>TFP', 1}];
   SS=[SS, zeros(size(SS,1),1); zeros(1, size(SS,1)), SS_TFP];
   EE=[EE; EE_TFP];
   LL_past_impact=[LL; LL_TFP*0];
   LL=[LL; LL_TFP];
else
   LL_past_impact=LL;
end

%% reduced form prior
if model_version==7410
    hyperparams.tau     =   1/4;             % 0.1 overall tightness */
    hyperparams.d       =   4;               % scaling down the variance for the coefficients of a distant lag */
    hyperparams.w       =   1;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparams.lambda  =   1/4;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparams.mu      =   1/4;               % 12 tuning parameter for the covariance between coefficients*/
    prior_only=0;
elseif model_version==7411
    hyperparams.tau     =   .5;             % 0.1 overall tightness */
    hyperparams.d       =   4;               % scaling down the variance for the coefficients of a distant lag */
    hyperparams.w       =   1;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparams.lambda  =   0.5;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparams.mu      =   0.5;               % 12 tuning parameter for the covariance between coefficients*/
    prior_only=0;
elseif model_version==7412
    hyperparams.tau     =   1;             % 0.1 overall tightness */
    hyperparams.d       =   4;               % scaling down the variance for the coefficients of a distant lag */
    hyperparams.w       =   1;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparams.lambda  =   1;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparams.mu      =   1;               % 12 tuning parameter for the covariance between coefficients*/
    prior_only=0;
else
    prior_only=-1;
    hyperparams.tau     =   0;             % 0.1 overall tightness */
    hyperparams.d       =   0;               % scaling down the variance for the coefficients of a distant lag */
    hyperparams.w       =   0;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparams.lambda  =   0;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparams.mu      =   0;               % 12 tuning parameter for the covariance between coefficients*/
end

time_all=numeric_data(:,strcmp('year', varnameAll));
beg_date    = find(time_all>1960,1);

if any(strcmp(varname, 'xxxFFR'))
    beg_date    = find(time_all>1984,1);
    end_date    = find(time_all==2007);
else
    end_date    = find(time_all==2015);
end
if START_1983
    beg_date    = find(time_all>=1983,1);
    end_date    = find(time_all==2016);
    model_version = str2double(['6' char(extractAfter(num2str(model_version),1))]);
end
if ~isequal(HR_MULTIPLE, 1)
     model_version = model_version*100+HR_MULTIPLE(end)*10;
end
time_ax     = time_all(beg_date:end_date);

ydata       = numeric_data(beg_date:end_date,data_index);
ydata(any(isnan(ydata),2),:)=[];
if model_version == 11
    Plot_Raw_Data;
end

[T,nvar]    = size(ydata); %nvar is the number of variables, T is the number of data points
if model_version>=10
    xdata       = [ones(T,1), (1:T)', ((1:T).^2)' ] ; %option to use if you want to use a constant only, no seasonal dummies
else
    xdata       = ones(T,1); %option to use if you want to use a constant only, no seasonal dummies
end

lags=4;

%% Graphics config
% use that they are conveniently ordered first
if ~any(cellfun(@isempty, strfind(IncludeSeriesNames,'Consumption')))
    NoMacroVars = 5;
else
    NoMacroVars = 4;
end
Grid = [1 9 25];
legendEntries = {'Impact'; '2 years out'; '6 years out'};
TimeLabel='Horizon (quarters)';
UnitLabel=cell(nvar,1);
UnitLabel(:)={'Percent'};
Plot_Horizon=24;
density_hh = 9;
MyXticks=5:4:Plot_Horizon+1;

%% PART TWO: MAKING POSTERIOR DRAWS
% -------------------------------------------------------------------------
ndraw_frac  = ndraws;
nlag        = lags+1;

Do_CumulativeMatrix=Do_Cumulative*ones(1,nvar); 

[By,Bx,omega]=rfvardraws_dummyobs(ydata,lags,xdata,ndraws,hyperparams, prior_only);

J           = [eye(nvar) zeros(nvar,nvar*(lags-1))];

NoTrials=5;

% Initialize
draw_i=0;
HR_rejected_draws=0;
SR_rejected_draws=0;

tmp_scale = 1;

% set invariant variables
ll      = max( reshape(cell2mat(LL(:)),1,[]) );
b       = cell(nshocks,1);
for ss=1:nshocks
    b{ss}       = zeros(nres(ss)*H,1);
end
Aeq     = []; 
beq     = []; 
lb      = []; 
ub      = []; 

options = optimset('Display', 'off', 'Algorithm', 'active-set', 'TolCon', 1e-10, 'TolFun', 1e-10, 'TolX', 1e-10, 'MaxIter', 5e3, 'MaxFunEvals', 5e3 ) ;

Empty_Std_Alg_SR=zeros(ndraws, 3, nshocks);
Empty_Std_Alg_HR=zeros(ndraws, 3, nshocks);

if TFP_Lambda~=0
    model_version = model_version*1e4+TFP_Lambda*10+7;
end
if ~DO_SLOPE_W_SR
    model_version = model_version*1e3+5;
end

%% compute posterior of lambdas
nlambdas = length(HR_MULTIPLE);
nonempty_pr = zeros(1, nlambdas+1);
LL_original = LL;
LL_past_impact_original = LL_past_impact ;
for ll=1:nlambdas
    LL{1} = LL_original{1}*HR_MULTIPLE(ll);
    LL_past_impact{1} = LL_past_impact_original{1}*HR_MULTIPLE(ll);
    IdentificationGivenReducedForm_SoftZero_CheckOnly;

    PosteriorChebychevFlag = ChebychevFlag;
    PosteriorAcceptanceRate = (mean(ChebychevFlag(2:end,:)==1 | ChebychevFlag(2:end,:)==3,1));
    nonempty_pr(ll+1) = PosteriorAcceptanceRate(2);
end
nonempty_pr(1) = PosteriorAcceptanceRate(1);
save prior_posterior_tmp;

%% Plot probability of non-empty identified set
FontSize = 12;
GridRGB = [0 0 0]+.3;

close all;

hndl=plot(HR_MULTIPLE,nonempty_pr(2:end), 'linewidth',2);

grid on

legend('off')
xlabel('lambda')
ylabel('probability of nonempty identified set')

grid on
set(findall(gcf, '-property', 'FontSize'), 'FontSize', FontSize, 'fontWeight', 'normal','FontName','Times New Roman');
set(findall(gcf,'type','axes'),'ycolor',GridRGB,'xcolor',GridRGB,'GridLineStyle','-','LineWidth',.5,'FontSize',FontSize);

if DO_SAVE
    print(gcf, '-depsc', '-r300',sprintf(['%sMultipleLambdas_Model_%d_nonempty.eps'],save_path_bound,model_version) )
end