function [YYdum, XXdum, breakss,YYact,XXact] = get_dummyobs(ydata,xdata,lags, hyperparameters)
%% Part 1: get vm_spec part
% =========================================================================

% Some specifications
% -------------------------------------------------------------------------
T0  = lags;               % size of pre-sample */
nex = size(xdata,2);               % number of exogenous vars; 1 means intercept only */

% Hyperparameters for the Minnesota prior
% -------------------------------------------------------------------------
if ~exist('hyperparameters', 'var')
    hyperparameters.tau     =   .5;             % 0.1 overall tightness */
    hyperparameters.d       =   4;               % scaling down the variance for the coefficients of a distant lag */
    hyperparameters.w       =   1;               % number of observations used for obtaining the prior for the covariance matrix of error terms*/
    hyperparameters.lambda  =   0.5;               % 2.3 tuning parameter for coefficients for constant*/
    hyperparameters.mu      =   0.5;               % 12 tuning parameter for the covariance between coefficients*/
end
%% Part 2: Dummy Observations
% =========================================================================
YY      = ydata;
nv      = size(YY,2);     %* number of variables */
nobs    = size(YY,1)-T0;  %* number of observations */

YY0     =   YY(1:T0,:);  
ybar    =   mean(YY0)';      
sbar    =   std(YY0)'; 
premom  =   [ybar sbar];
hyp     = [hyperparameters.tau; hyperparameters.d; hyperparameters.w; hyperparameters.lambda; hyperparameters.mu];

% tau:      dummies for the coefficients of the first lag
% hyperparameters.d:        dummies for the coefficients of the remaining lags
% w:        dummies for the covariance matrix of error terms
% lambda:   dummies for the coefficents of deterministic component (constant,lenear trend, quadratic trend,...)
% mu:       dummies for the covariance matrix of coefficients of different lags

[YYdum, XXdum, breakss] = varprior_h(nv,lags,nex,hyp,premom);


%% Part 3: Posterior Sampling
% =========================================================================

% Actual observations
YYact = YY(T0+1:T0+nobs,:);
XXact = zeros(nobs,nv*lags);    
i = 1;
while (i <= lags)
    XXact(:,(i-1)*nv+1:i*nv) = YY(T0-(i-1):T0+nobs-i,:);
    i = i+1;
end

% last column of XXact = constant
XXact = [XXact [ones(nobs,1), (1:nobs)', ((1:nobs).^2)' ]];




