function [SS, EE, H_bounds, Do_Cumulative, FAVAR_IDX, nres, nvars, ncore, nshocks, SS_combined, EE_combined] = ...
    my_EnglishToMatrices_Slopes_cell(cell_EnglishRestrictions, VarLabels, CoreVars)
% translate cell of cells with restrictions to matrices such that
% -SS{ii}*(EE{ii}-LL{ii})*[A^(h(1)-1)-A^(h(2)-1)]*Cholesky <=0
%
% FAVAR_IDX: Column 1: Maps from VarLabels to FAVAR index
%            Column 2: Maps from FAVAR index to VarLabels (inverse of column 1)

% For convenience
nshocks = length(cell_EnglishRestrictions);
nvars= size(VarLabels,1);
ncore= size(CoreVars,1);
H_bounds = cell(nshocks,1);

% match only on trimmed lowercase variable names
VarLabels = strtrim(lower(VarLabels));
if ~isempty(CoreVars)
    CoreVars  = strtrim(lower(CoreVars));
else
    CoreVars  = {};
end;

% delete core variables from VarLabels
[~,IA] = setdiff(VarLabels, CoreVars, 'stable');
FactorVarLabels = VarLabels(IA);

FAVAR_IDX = zeros(nvars,2);
for nn=1:nvars
    FAVAR_IDX(nn,1) = LabelToIndex(VarLabels{nn}, FactorVarLabels, CoreVars, ncore, nvars);
end;
for nn=1:nvars
    FAVAR_IDX(nn,2) = LabelToIndex(VarLabels{nn}, VarLabels(FAVAR_IDX(:,1)), {}, 0, nvars);
end;

nres = zeros(nshocks,1);
SS = cell(nshocks,1);
EE = cell(nshocks,1);
% LL = cell(nshocks,1);
Do_Cumulative = cell(nshocks,1);
for ss=1:nshocks
    nres(ss,1) = size(cell_EnglishRestrictions{ss},1);
    if nres(ss,1)>0
        Do_Cumulative{ss} = cell2mat(cell_EnglishRestrictions{ss}(:,2));
        % Read signs
        SS{ss} = diag(cell2mat(cell_EnglishRestrictions{ss}(:,3)));
        % Read restriction horizons
        H_bounds{ss}(:,1) = cell2mat(cell_EnglishRestrictions{ss}(:,4));
        H_bounds{ss}(:,2) = cell2mat(cell_EnglishRestrictions{ss}(:,5));
        % initialize
        EE{ss} = zeros(nres(ss,1),nvars);


        for rr=1:nres(ss,1)
            LHS_VarLabel = strtrim(lower(cell_EnglishRestrictions{ss}{rr,1}));

            EE{ss}(rr,  LabelToIndex(LHS_VarLabel, FactorVarLabels, CoreVars, ncore, nvars)) = 1;
        end;
    else
        Do_Cumulative{ss} = [];
        SS{ss} = [];
        H_bounds{ss}  = [];
        EE{ss} = [];
    end;
end;

% produce large matrix that premultiplies the IRFs that are post-multiplied by vec(Q_identfied)
SS_combined = zeros(sum(nres),sum(nres));
EE_combined = zeros(sum(nres),nshocks *nvars);
Do_Cumulative_combined =  zeros(sum(nres),1 );
for ss=1:nshocks
    SS_combined(sum(nres(1:ss-1))+(1:nres(ss)), sum(nres(1:ss-1))+(1:nres(ss)) ) = SS{ss};
    EE_combined(sum(nres(1:ss-1))+(1:nres(ss)),(ss-1)*nvars+(1:nvars) ) = EE{ss};
    Do_Cumulative_combined(sum(nres(1:ss-1))+(1:nres(ss)),1) = Do_Cumulative{ss};
end;


function idx = LabelToIndex(ObjectOfInterest, FactorVarLabels, CoreVars, ncore, nvars)

if sum(strcmp(ObjectOfInterest, CoreVars))==1
    % variable is in core of the VAR
    idx = nvars - ncore + find(strcmp(ObjectOfInterest, CoreVars));
else
    idx = find(strcmp(ObjectOfInterest, FactorVarLabels));
end;