function [SS, EE, LL, H_bounds, Do_Cumulative, FAVAR_IDX, nres, nvars, ncore, nshocks, SS_combined, EE_combined, LL_combined] = ...
    my_EnglishToMatrices_cell(cell_EnglishRestrictions, VarLabels, CoreVars)
% translate cell of cells with restrictions to matrices such that
% -SS{ii}*(EE{ii}-LL{ii})*[Loading*Factor; CoreVars] <= 0 
% if H_bounds(:,1) <= h <= Hbounds(:,2)
%
% FAVAR_IDX: Column 1: Maps from VarLabels to FAVAR index
%            Column 2: Maps from FAVAR index to VarLabels (inverse of column 1)

% For convenience
nshocks = length(cell_EnglishRestrictions);
nvars   = size(VarLabels,1);
ncore   = size(CoreVars,1);
H_bounds = cell(nshocks,1);


% match only on trimmed lowercase variable names
VarLabels = strtrim(lower(VarLabels));
if ~isempty(CoreVars)
    CoreVars  = strtrim(lower(CoreVars));
else
    CoreVars  = {};
end

% delete core variables from VarLabels
[~,IA] = setdiff(VarLabels, CoreVars, 'stable');
FactorVarLabels = VarLabels(IA);



FAVAR_IDX = zeros(nvars,2);
for nn=1:nvars
    FAVAR_IDX(nn,1) = LabelToIndex(VarLabels{nn}, FactorVarLabels, CoreVars, ncore, nvars);
end;
for nn=1:nvars
    FAVAR_IDX(nn,2) = LabelToIndex(VarLabels{nn}, VarLabels(FAVAR_IDX(:,1)), {}, 0, nvars);
end;

nres = zeros(nshocks,1);
SS = cell(nshocks,1);
EE = cell(nshocks,1);
LL = cell(nshocks,1);
Do_Cumulative = cell(nshocks,1);
for ss=1:nshocks
    nres(ss,1) = size(cell_EnglishRestrictions{ss},1);
    % Read restriction horizons
    if size(cell_EnglishRestrictions{ss})<4
        H_bounds{ss} = nan(nvars, 2); 
    else
        NonEmptyIdx = (~cellfun( 'isempty', cell_EnglishRestrictions{ss}(:,3) )) & (~cellfun( 'isempty', cell_EnglishRestrictions{ss}(:,4) ));
        H_bounds{ss}(NonEmptyIdx,:) = cell2mat( cell_EnglishRestrictions{ss}(NonEmptyIdx, 3:4) );
    end;    
    Do_Cumulative{ss} = cell2mat(cell_EnglishRestrictions{ss}(:,2));
    % initialize
    SS{ss} = zeros(nres(ss,1),nres(ss,1));
    EE{ss} = zeros(nres(ss,1),nvars);
    LL{ss} = zeros(nres(ss,1),nvars);


    for rr=1:nres(ss,1)
        % "<" or ">" 
        sfind_ge = strfind(cell_EnglishRestrictions{ss}{rr,1}, '>');
        if isempty(sfind_ge); sfind_ge=0; end;
        sfind_le = strfind(cell_EnglishRestrictions{ss}{rr,1}, '<');
        if isempty(sfind_le); sfind_le=0; end;

        if  sfind_ge >0 && sfind_le==0
           CurrentSign = 1;
           CurrentPos  = strfind(cell_EnglishRestrictions{ss}{rr,1}, '>');
        elseif sfind_le >0 && sfind_ge==0
           CurrentSign=-1;
           CurrentPos  = strfind(cell_EnglishRestrictions{ss}{rr,1}, '<');
        else
            error('must have exactly one strict restriction');
        end;
        LHS_VarLabel = strtrim(lower(cell_EnglishRestrictions{ss}{rr,1}(1:CurrentPos-1)));
        RHS_VarLabel = strtrim(lower(cell_EnglishRestrictions{ss}{rr,1}(CurrentPos+1:end)));

        if isequal(RHS_VarLabel,'0') % there shouldn't be zeros on both sides of the inequality
            EE{ss}(rr,  LabelToIndex(LHS_VarLabel, FactorVarLabels, CoreVars, ncore, nvars)) = 1;
            CurrentSign = -CurrentSign;
        else
            EE{ss}(rr, LabelToIndex(LHS_VarLabel, FactorVarLabels, CoreVars, ncore, nvars)) = 1;

            % if <>0, then heterogeneity restriction
            LL{ss}(rr, LabelToIndex(RHS_VarLabel, FactorVarLabels, CoreVars, ncore, nvars)) = 1;
    %         else
    %             LL(rr,strcmp(LHS_VarLabel,VarLabels)) = 1;
    %         end;
            CurrentSign = -CurrentSign;
        end;
        SS{ss}(rr,rr) = -CurrentSign; % change signs so that: -SS * (EE - LL) * IRF <= 0
    end;

end;

% produce large matrix that premultiplies the IRFs that are post-multiplied by vec(Q_identfied)
SS_combined = zeros(sum(nres),sum(nres));
EE_combined = zeros(sum(nres),nshocks *nvars);
LL_combined = zeros(sum(nres),nshocks *nvars);
Do_Cumulative_combined =  zeros(sum(nres),1 );
for ss=1:nshocks
    SS_combined(sum(nres(1:ss-1))+(1:nres(ss)), sum(nres(1:ss-1))+(1:nres(ss)) ) = SS{ss};
    EE_combined(sum(nres(1:ss-1))+(1:nres(ss)),(ss-1)*nvars+(1:nvars) ) = EE{ss};
    LL_combined(sum(nres(1:ss-1))+(1:nres(ss)),(ss-1)*nvars+(1:nvars) ) = LL{ss};    
    Do_Cumulative_combined(sum(nres(1:ss-1))+(1:nres(ss)),1) = Do_Cumulative{ss};
end;


function idx = LabelToIndex(ObjectOfInterest, FactorVarLabels, CoreVars, ncore, nvars)

if sum(strcmp(ObjectOfInterest, CoreVars))==1
    % variable is in core of the VAR
    idx = nvars - ncore + find(strcmp(ObjectOfInterest, CoreVars));
else
    idx = find(strcmp(ObjectOfInterest, FactorVarLabels));
end;