function [c, norm_f, c_grad, gradient] = my_eq_constraint_v2(z, nvar, nshocks)
% inequality constraint, equality constraint, and corresponding gradients
% nvar = number of variables

c=[];
c_grad=[];
x=z(1:nvar);
switch nshocks
    case 1
        f = x'*x-1;
        grad_f = 2*x;
        norm_f= f*f';

        gradient = 2*grad_f*f';
        
    case 2
        y=z(nvar+1:2*nvar);
        % function: the three lower triangular elements of [x,y]'[x,y]=I_2;
        f = [x'*x-1, x'*y,y'*y-1]; % 1 x 3
        grad_f = [2*x, y,zeros(nvar,1); zeros(nvar,1), x, 2*y]; % N x 3

        norm_f= f*f';

        gradient = 2*grad_f*f';
    case 3
        y=z(nvar+1:2*nvar);
        yy=z(nvar*2+1:3*nvar);
        % function: the three lower triangular elements of [x,y]'[x,y]=I_2;
        f = [x'*x-1, x'*y,x'*yy, y'*yy, y'*y-1, yy'*yy-1]; % 1 x 3
        grad_f = [2*x, y, yy, zeros(nvar,3); ...
                    zeros(nvar,1), x, zeros(nvar,1), yy, 2*y, zeros(nvar,1); ...
                    zeros(nvar,2), x, y, zeros(nvar,1), 2*yy; ]; % N x 3

        norm_f= f*f';

        gradient = 2*grad_f*f';
    otherwise
        error('not set up for more than three shocks')
end
