function merged_res_lbls = my_f_format_labels(RestrictionLabels, SlopeRestrictions, Dictionary)
% input: cell-arrays with the individual restrictions
% optional input: cellstring dictionary to shorten restriction labels
% (column 1: what to substitute, column 2: what to substitute it with)
% output: one cell-array that has all restriction labels for shock i before
% those for shock j, j>i.
% restriction horizons are currently discarded.
% this format works for my_f_res_no_to_res_lbl
%
% Example: 
% merged_res_lbls = my_f_format_labels(RestrictionLabels, SlopeRestrictions, {'Employment','Emp.'; 'Confidence', 'Conf'; 'Consumers', 'Cons'})

merged_res_lbls = {};
nshocks = length(RestrictionLabels);
for ss=1:nshocks
    if ~isempty(SlopeRestrictions{ss})
        FormattedSlopeRestrictions = SlopeRestrictions{ss}(:,1);
        for tt = 1:size(SlopeRestrictions{ss}(:,1), 1)
            if cell2mat(SlopeRestrictions{ss}(tt,3)) == 1 % increasing slope
                FormattedSlopeRestrictions{tt} = [FormattedSlopeRestrictions{tt} ' rises'];
            else
                FormattedSlopeRestrictions{tt} = [FormattedSlopeRestrictions{tt} ' falls'];
            end;
        end;
    else
        FormattedSlopeRestrictions = {};
    end;
%     merged_res_lbls = [merged_res_lbls(:); RestrictionLabels{ss}(:,1); FormattedSlopeRestrictions];
    merged_res_lbls = [merged_res_lbls(:); ...
            cellstr([repmat(num2str(ss,'s%1.0f:') , size(RestrictionLabels{ss}(:,1),1) + size(FormattedSlopeRestrictions, 1) ,1), char([RestrictionLabels{ss}(:,1); FormattedSlopeRestrictions])]) ];
end;

if exist('Dictionary', 'var')
    for ss=1:size(Dictionary, 1)
        merged_res_lbls  = regexprep(merged_res_lbls , Dictionary{ss,1}, Dictionary{ss,2});
    end;
end;