function [map_matrix, shock_vector, LIST_OF_NO] = my_f_res_no_to_res_lbl( LIST_OF_NO, H_bounds, H_bounds_sl )
% write function that maps restriction number to restriction label
% Shock by shock, standard restrictions come first, slope restrictions last
% Complements my_f_format_labels.m

% Slope restrictions of the same type are not aggregated, but left separately

% sort list to make sure the indexing via "ismember" works
if ~issorted(LIST_OF_NO)
    LIST_OF_NO = sort(LIST_OF_NO);
    fprintf(2, 'list of restrictions is not sorted -- output is based on sorted list \n');
end;

N_list   = size(LIST_OF_NO(:),1);
nshocks = length(H_bounds);
N_res = zeros(nshocks,1);
N_res_sl = zeros(nshocks,1);
for ss=1:nshocks
    N_res(ss)    = size(H_bounds{ss},1);
    N_res_sl(ss) = size(H_bounds_sl{ss},1);
end;
map_matrix = zeros(sum(N_res) + sum(N_res_sl), N_list); % Restriction label idx x no of restrictions to be labeled
shock_vector = zeros(N_list, 1); % Shock label x no of restrictions to be labeled

res_total_count = 0;
res_ind_count = nan(nshocks,1);
for ss=1:nshocks
    H_min = min(H_bounds{ss}(:,1));
    H_max = max(H_bounds{ss}(:,2));
    for hh=H_min:H_max
        h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
        res_total_count = res_total_count + sum(h_idx);
    end;
    res_total_count = res_total_count + N_res_sl(ss);
    if ss>1
        res_ind_count(ss) = res_total_count - sum(res_ind_count(1:ss-1)) ;
    else
        res_ind_count(1) = res_total_count ;
    end;
end;

if res_total_count < max(LIST_OF_NO)
    error('more restrictions in input than total restrictions');
end;

if max(N_res_sl)>1
   fprintf(2, 'Slope restrictions of the same type are not aggregated, but left separately \n') ;
end

% Restriction matrix is constructed by first taking care of all
% restrictions for one shock, including slope restrictions (if any)

% Slope restrictions of the same type are not aggregated, but left
% separately
new_total_count = 0;
previous_res = 0;
previous_res_sl = 0;
StartCount = 0;
for ss=1:nshocks
    if ss>1
        previous_res = sum(N_res(1:ss-1));
        previous_res_sl = sum(N_res_sl(1:ss-1));
    end;
%     RelevantList = LIST_OF_NO(StartCount + (1:res_ind_count(ss)));
    RelevantList = LIST_OF_NO( LIST_OF_NO>=(StartCount + 1) & LIST_OF_NO<=max(StartCount+res_ind_count(ss)) );
    % all currently relevant shocks have label no ss
    shock_vector( ismember(LIST_OF_NO,RelevantList) ) = ss;
    % For a given shock, slope restrictions come last.
    % Restrictions are ordered by label, if they apply to a given horizon
    H_min = min(H_bounds{ss}(:,1));
    H_max = max(H_bounds{ss}(:,2));
    for hh=H_min:H_max
        h_idx = H_bounds{ss}(:,1)<=hh & hh<=H_bounds{ss}(:,2);
        % isolate the restrictions that are relevant right now
        LIST_IDX= new_total_count < RelevantList & RelevantList <= new_total_count + sum(h_idx);
%         LIST_ENTRIES = NaN(sum(h_idx));
%         for ii=find(h_idx')
%             LIST_ENTRIES(ii) = 1;
%         end;
%         LIST_ENTRIES = RelevantList(new_total_count < RelevantList & RelevantList <= new_total_count + find(h_idx));
        LIST_ENTRIES = RelevantList(new_total_count < RelevantList & RelevantList <= new_total_count + sum(h_idx));
        % check for their type
%         TYPE_IDX = find(H_bounds{ss}(LIST_ENTRIES-new_total_count,1)<=hh & hh<=H_bounds{ss}(LIST_ENTRIES-new_total_count,2));
        TYPE_IDX = find(H_bounds{ss}(h_idx,1)<=hh & hh<=H_bounds{ss}(h_idx,2));
        %need function here that maps array to array
        map_matrix(  previous_res + previous_res_sl + find(h_idx), ismember(LIST_OF_NO(:),LIST_ENTRIES(:)) )=eye(sum(h_idx), sum(ismember(LIST_OF_NO(:),LIST_ENTRIES(:))) );
%         idx = sub2ind(size(map_matrix),  previous_res + previous_res_sl + TYPE_IDX , find(ismember(LIST_OF_NO(:),LIST_ENTRIES(:)) ) );
%         map_matrix( previous_res + previous_res_sl + TYPE_IDX , ismember(LIST_OF_NO,LIST_ENTRIES)  )=1;
%         map_matrix( idx )=1;
%         new_total_count = new_total_count + sum(h_idx);
        new_total_count = new_total_count + sum(h_idx);
    end;
    % if there are any remaining entries, they are slope restrictions
    % perhaps this could be written more simply as
    % max(LIST_ENTRIES-new_total_count, 0)
%     LIST_IDX= new_total_count < RelevantList & RelevantList <= new_total_count + N_res_sl(ss);
    LIST_ENTRIES = RelevantList(new_total_count < RelevantList & RelevantList <= new_total_count + N_res_sl(ss));
%     TYPE_IDX = size(H_bounds{ss},1) + find(LIST_ENTRIES-new_total_count<=N_res_sl(ss) & (LIST_ENTRIES>new_total_count));
    TYPE_IDX = size(H_bounds{ss},1) + (LIST_ENTRIES-new_total_count) * any(LIST_ENTRIES-new_total_count<=N_res_sl(ss) & (LIST_ENTRIES>new_total_count));
%     idx = sub2ind(size(map_matrix),  previous_res + previous_res_sl + TYPE_IDX , find(ismember(LIST_OF_NO(:),LIST_ENTRIES(:)) ) );
    map_matrix( previous_res + previous_res_sl + TYPE_IDX , ismember(LIST_OF_NO,LIST_ENTRIES) )=eye(length(TYPE_IDX), sum(ismember(LIST_OF_NO,LIST_ENTRIES)));
%     map_matrix( idx )=1;
    
    StartCount = StartCount + res_ind_count(ss);
    new_total_count = new_total_count + N_res_sl(ss);
end;
