% set up model data and restrictions
sr_version      = 2;     % 1: SR on level, 2: SR on GR. % fixed across all applications
DO_FERNALD = 1; % fixed across all applications
ONLY_ALG2=false; % currently needed in code, although conceptually easy to relax
DO_ALT_TRUNC_N=0;
[numeric_data,txt_data,raw_data]    = xlsread([load_path file_name]); 
varnameAll= txt_data(1,1:end)';
save('PCE_IP_Data.mat', 'numeric_data', 'txt_data', 'raw_data', 'varnameAll');
if ~isequal(length(varnameAll), size(numeric_data,2))
    error('data does not match up');
end

switch model_version
    case {1,11,113,101,102, 31}
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... 
            'dlogConfidence', 'Confidence'; ...
            'dlogEmployment', 'Employment'; ...
            'dlogcum_rtn1', 'Consumers'; ...
            'dlogcum_rtn2', 'Manu'; ...
            'dlogcum_rtn3', 'HiTec'; ...
            'dlogcum_rtn4', 'Health'; ...
            'dlogcum_rtn5', 'Other';...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
%         if model_version==1 || model_version==101 || model_version==11 || model_version==31 
            IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
%         end
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);
%         data_index  = [1:14 16 17]; % IP, Pharma, Plastics, Cars, Leather, Pottery, Tobacco, CPI, NBR, FFR
        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

        % Added restriction: Boom is stronger than actual TFP shock
        % Equivalent to the "soft zero" restriction, but with a loading of 1!

        RestrictionLabels{1} = { ... Restriction, do cumulative IRF, from horizon, to horizon
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1, 1+H-1; ...      2
                'Confidence>0',1, 1, 1+H-1 ; ...      2
                'Employment>0',1, 1, 1+H-1; ...            3        
                ...
                'Consumers>0',1, 1, 1+H-1; ...            3
                'Other>0',1, 1, 1+H-1; ...            4
                'Manu>Consumers',1, 1, 1+H-1; ...            3
                'Manu>Other',1, 1, 1+H-1; ...            4
                'Health>Manu',1, 1, 1+H-1;      ...  7
                'HiTec>Manu',1, 1, 1+H-1;      ...  7
            };

        if model_version==113
            SlopeRestrictions{1} = { ...
                'TFP', 1,1, 1, H; ...
                };
        else
            SlopeRestrictions{1} = { ...
            %    'TFP', 1,1, 1, H; ...
                };
        end
            if model_version==31
                RestrictionLabels(strcmp('Employment>0', RestrictionLabels),:)=[];
            end

        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end

    case {666}
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... 
            'dlogConfidence', 'Confidence'; ...
            'dlogEmployment', 'Employment'; ...
            'dlogcum_rtn1', 'Consumers'; ...
            'dlogcum_rtn2', 'Manu'; ...
            'dlogcum_rtn3', 'HiTec'; ...
            'dlogcum_rtn4', 'Health'; ...
            'dlogcum_rtn5', 'Other';...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
        IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);
        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

        % Added restriction: Boom is stronger than actual TFP shock
        % Equivalent to the "soft zero" restriction, but with a loading of 1!
        RestrictionLabels{1} = { ...
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1, 1+H-1; ...      2
                'Confidence>0',1, 1, 1+H-1 ; ...      2
                'Employment>0',1, 1, 1+H-1; ...            3        
                ...
                'Health>0',1, 1, 1+H-1; ...            3
                'HiTec>0',1, 1, 1+H-1; ...            4
                'Manu>Health',1, 1, 1+H-1; ...            3
                'Manu>HiTec',1, 1, 1+H-1; ...            4
                'Consumers>Manu',1, 1, 1+H-1;      ...  7
                'Other>Manu',1, 1, 1+H-1;      ...  7
            };
 

 
        SlopeRestrictions{1} = { ...
            };

        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end
        end

    case {74, 7401, 7402, 7410, 7411, 7412} %small macro only VAR
        if ~START_1983
            error('For this model we use the post-Volcker sample');
        end
        
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... 
            'rP', 'Stocks'; ...
            'GS2', 'FFR'; ...
            'logCPI', 'Prices'; ...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
       
        % Alternative interest rates to deal with ELB
        if model_version==7401
            IncludeSeries = regexprep(IncludeSeries, 'GS2', 'Krippner');            
        end
        if model_version==7402
            IncludeSeries = regexprep(IncludeSeries, 'GS2', 'Wu');            
        end
        
        IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);

        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

        % Added restriction: Boom is stronger than actual TFP shock
        % Equivalent to the "soft zero" restriction, but with a loading of 1!
        RestrictionLabels{1} = { ...
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1, 1+H-1; ...      2
                'Stocks>0',1, 1, 1+H-1 ; ...      2
            };

        RestrictionLabels{2} = { ...
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1,1+H-1; ...      2
                'Prices<0',1, 1, 1+H-1; ...            3        
            };


        RestrictionLabels{3} = { ...
                'Output<0' ,1, 1, 1+H-1; ...           1
                'FFR>0',1, 1, 1+3-1; ...            3        
                'Prices<0',1, 1, 1+3-1; ...            3        
            };

        SlopeRestrictions{1} = { ...
           'TFP', 1,1, 1, 2; ... initially increasing
           'TFP', 1,1, 2, 3; ... initially increasing
            };
        SlopeRestrictions{2} = { ...
           'TFP', 1,-1, 1, 2; ... initially decreasing
           'TFP', 1,-1, 2, 3; ... initially decreasing
            };
        SlopeRestrictions{3} = { ...
           'FFR', 1,-1, 1, 2; ... initially decreasing
           'FFR', 1,-1, 2, 3; ... initially decreasing
            };

    
        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
%             Do_Cumulative_slope=Do_Cumulative_slope*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end
    case 749 
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... 
            'rP', 'Stocks'; ...
            'GS2', 'FFR'; ...
            'logCPI', 'Prices'; ...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
        
        IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);
        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

        % Added restriction: Boom is stronger than actual TFP shock
        % Equivalent to the "soft zero" restriction, but with a loading of 1!
        RestrictionLabels{1} = { ...
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1, 1+H-1; ...      2
                'Stocks>0',1, 1, 1+H-1 ; ...      2
            };


        SlopeRestrictions{1} = { ...
           'TFP', 1,1, 1, 2; ... initially increasing
           'TFP', 1,1, 2, 3; ... initially increasing
            };

   
        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end
        
    case 748 
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... 
            'rP', 'Stocks'; ...
            'FEDFUNDS', 'FFR'; ...
            'logCPI', 'Prices'; ...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
        
        IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);
%         data_index  = [1:14 16 17]; % IP, Pharma, Plastics, Cars, Leather, Pottery, Tobacco, CPI, NBR, FFR
        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

        % Added restriction: Boom is stronger than actual TFP shock
        % Equivalent to the "soft zero" restriction, but with a loading of 1!

        RestrictionLabels{1} = { ...
                'Output>0' ,1, 1, 1+H-1; ...           1
                'TFP>0',1 , 1,1+H-1; ...      2
                'Prices<0',1, 1, 1+H-1; ...            3        
            };

        SlopeRestrictions{1} = { ...
           'TFP', 1,-1, 1, 2; ... initially decreasing
           'TFP', 1,-1, 2, 3; ... initially decreasing
            };

    
        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
%             Do_Cumulative_slope=Do_Cumulative_slope*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end
    case 747 %inventories
        IncludeSeries = {...
            'dlogRealGDP', 'Output'; ...
            'dlogTFP', 'TFP'; ... housing
            'rP', 'Stocks'; ...
            'FEDFUNDS', 'FFR'; ...
%             'infl', 'Inflation'; ...
            'logCPI', 'Prices'; ...
%             'dlogUtilization', 'Utilization'; ...
            };
        if DO_FERNALD
            IncludeSeries = regexprep(IncludeSeries, 'dlogEmployment', 'dlogHours');            
            IncludeSeries = regexprep(IncludeSeries, 'dlogRealGDP', 'dlogY');            
        end
        
        IncludeSeries = regexprep(IncludeSeries, 'dlog', 'log');
        data_index=zeros(length(IncludeSeries),1);
        for ii=1:length(IncludeSeries)
            data_index(ii) = find(strcmp(IncludeSeries{ii,1}, varnameAll));
        end
        IncludeSeriesNames = IncludeSeries(:,2);
%         data_index  = [1:14 16 17]; % IP, Pharma, Plastics, Cars, Leather, Pottery, Tobacco, CPI, NBR, FFR
        nvar=length(data_index);
        save_name   = sprintf('PCE%d_SRMode%d',model_version,sr_version);
        varname     = IncludeSeries(:,2);
        posSR_index = [1];
        negSR_index = [2:nvar];
        
        SS=eye(nvar);

% Added restriction: Boom is stronger than actual TFP shock
% Equivalent to the "soft zero" restriction, but with a loading of 1!
RestrictionLabels{1} = { ...
        'Output<0' ,1, 1, 1+H-1; ...           1
        'FFR>0',1, 1, 1+3-1; ...            3        
        'Prices<0',1, 1, 1+3-1; ...            3        
    };
  
SlopeRestrictions{1} = { ...
   'FFR', 1,-1, 1, 2; ... initially decreasing
   'FFR', 1,-1, 2, 3; ... initially decreasing
    };

    
        nshocks = length(RestrictionLabels);
        Do_Cumulative=ones(nvar,1);
        if nshocks>=nvar 
            error('cannot handle as many shocks as variables: the Gibbs sampler breaks down');
        end
        [SS,EE,LL, H_bounds, Do_Cumulative_R, FAVAR_IDX, nres, ~, ~, ~, SS_combined, EE_combined, LL_combined] ...
            = my_EnglishToMatrices_cell(RestrictionLabels,IncludeSeriesNames,[]);
        [SS_slope,EE_slope,H_bounds_slope, Do_Cumulative_R_slope, FAVAR_IDX_slope, nres_slope, ~, ~, ~, SS_combined_slope, EE_combined_slope] ...
            = my_EnglishToMatrices_Slopes_cell(SlopeRestrictions,IncludeSeriesNames,[]);
        Do_CumulativeMatrix_R = cell(nshocks,1);
        Do_CumulativeMatrix_R_slope = cell(nshocks,1);
        H_max = -1;
        H_min = inf;
        for ss=1:nshocks
            H_max = max(H_max , max(H_bounds{ss}(:,2)));
            H_min = min(H_min, min(H_bounds{ss}(:,1)));
            Do_CumulativeMatrix_R{ss}=Do_Cumulative_R{ss}*ones(1,nvar);
            Do_CumulativeMatrix_R{ss}=Do_CumulativeMatrix_R{ss}*0;
            Do_Cumulative=Do_Cumulative*0;
            Do_CumulativeMatrix_R_slope{ss}=Do_Cumulative_R_slope{ss}*ones(nres_slope(ss)>0,nvar);
            Do_CumulativeMatrix_R_slope{ss}=Do_CumulativeMatrix_R_slope{ss}*0;
            fprintf(2,'currently cumulative matrix set to zero -- we are always estimating in levels \n')
        end
    otherwise
        error('currently not implemented');
end
