function [c,r,Result] = recursive_chebycenter(cell_of_As, AsCombined)
% Compute Chebyshev center of polytope A{i}x{i} <= 0
% and x{i} \in Null([x{1}, ..., x{i-1}])
%  r0 acceptable threshold (lower) for r. Optional.
% Requires optimization toolbox
% Result = 1: Sufficient condition satisfied;
% Result = 2: Necessary condition satisfied;
% Otherwise Result = -1.


N = size(cell_of_As{1},2);
M = length(cell_of_As); % no of shocks to be identified
PossibleOrderings=perms(1:M);
OrderN = factorial(M);

Result = -1; % initialize

oo = 0;
ValidPointFound=false;
c = zeros(N, M); % Chebychev centers
r = 0; % minimum radius
% b_base = zeros(N,1);

CommonTol = 1e-8;
% check if there is any ordering in which we can find a Chebychev center
% that is orthogonal to all prior centers -- sufficient condition
while oo<OrderN && ~ValidPointFound
    oo=oo+1;
    nn = 0;
    current_c = zeros(N,M);
    current_min_r = 100;
    ValidSoFar = true;
    while nn<M && ValidSoFar
        nn = nn+1;
        ii=PossibleOrderings(oo,nn) ;
        Kx=null(current_c(:,PossibleOrderings(oo,1:nn-1) )'); % pick previous shocks
        Padded_A = [cell_of_As{ii}; -eye(N,N); eye(N,N)];
        Padded_b = [zeros(size(cell_of_As{ii},1),1); ones(2*N,1)];
        [current_x, current_r] = chebycenter(Padded_A*Kx ,Padded_b ,1);
        current_min_r = min(current_min_r, current_r);
        if current_min_r <CommonTol || any(cell_of_As{ii}*Kx*current_x >0 + CommonTol)
            ValidSoFar = false;
        else
            current_c(:, ii) = Kx * current_x;
        end;
    end
    if ValidSoFar && current_min_r>0
        ValidPointFound = true;
        c = current_c;
        r = current_min_r;
        Result = 1;
    end;
end;

% If the sufficient condition is violated, check as a necessary condition
% that we can find centers ignoring the orthogonality condition
if ~ValidPointFound
    current_min_r = 100;
    ii=0;
    ValidSoFar=true;
    while ii<M && ValidSoFar
        ii=ii+1;
        Padded_A = [cell_of_As{ii}; -eye(N,N); eye(N,N)];
        Padded_b = [zeros(size(cell_of_As{ii},1),1);  ones(2*N,1)];
        [current_x, current_r] = chebycenter(Padded_A,Padded_b ,1);
        if current_r<1e-8 || any(cell_of_As{ii}*current_x >0 + 1e-8)
            ValidSoFar = false;
        else
            current_c(:, ii) = current_x;
            current_min_r = min(current_min_r, current_r);
        end;
    end;
    if ValidSoFar
        Result = 2;
        c = current_c;
        r = 0;        
        % now check if we can do even better than the necessary condition
        my_options_num = optimoptions('fmincon','Display','off','Algorithm','active-set', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e3, 'MaxFunctionEvaluations', 1e6, ...
            'UseParallel', false, 'SpecifyConstraintGradient', false, 'SpecifyObjectiveGradient', false);
        
        my_options_sqp = optimoptions('fmincon','Display','off','Algorithm','sqp', 'ConstraintTolerance', CommonTol, 'FunctionTolerance', CommonTol, 'StepTolerance', CommonTol, 'MaxIterations', 1e5, 'MaxFunctionEvaluations', 1e6, ...
            'UseParallel', false, 'SpecifyConstraintGradient', false, 'SpecifyObjectiveGradient', false);
%         sel_var         = 1;
%         sel_vec         = zeros(1,nvar);
%         sel_vec(sel_var)= 1;
%         obj_fun = @(x) get_irf_TD_mshocks(sel_vec, eye(N), x, nshocks, N, 1);
        my_eq_cons = @(z) my_eq_constraint_v2(z, N, M);
        b_combined = zeros(size(AsCombined,1),1);
        ones_combined = ones(N*M,1);
        try
            [x, ~, min_exit_flag] = fmincon(@(x) -norm(x) , current_c(:), AsCombined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num  );
            x = reshape(x,N,M)./repmat(sqrt(sum(reshape(x,N,M).^2,1)), N,1);
        catch ME
            [x, ~, min_exit_flag] = fmincon(@(x) -norm(x) , current_c(:), AsCombined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp  );
            x = reshape(x,N,M)./repmat(sqrt(sum(reshape(x,N,M).^2,1)), N,1);
        end;
        [~,eq_const_val]=my_eq_cons(x(:));        
        if all(AsCombined*x(:) <= b_combined + CommonTol) && eq_const_val<CommonTol
            min_exit_flag=9;
        end;
        ii=0;
        while ii<10 && min_exit_flag<=0
            ii=ii+1;
            addon = randn(N,M);
            x_start = (x+addon)./repmat(sqrt(sum((x+addon).^2,1)), N,1);
%             raw_c = randn(N,N);
%             [QQ,RR]=qr(raw_c);
%             QQ=QQ*diag(sign(diag(RR)));
            try
                [x, ~, min_exit_flag] = fmincon(@(x) -norm(x) , x_start(:), AsCombined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_num  );
                x = reshape(x,N,M)./repmat(sqrt(sum(reshape(x,N,M).^2,1)), N,1);
            catch ME
                [x, ~, min_exit_flag] = fmincon(@(x) -norm(x) , x_start(:), AsCombined, b_combined , [],[], -ones_combined , ones_combined , my_eq_cons,my_options_sqp  );
                x = reshape(x,N,M)./repmat(sqrt(sum(reshape(x,N,M).^2,1)), N,1);
            end;
            [~,eq_const_val]=my_eq_cons(x(:));        
            if all(AsCombined*x(:) <= b_combined + CommonTol) && eq_const_val<CommonTol
                min_exit_flag=9;
            end;
        end;
        if min_exit_flag>0
           Result = 3;
           c = reshape(x(:), N,M);
           r = 999;
        end
    end;
end;
